/**
 * SWE KEx-DatexII (de.bsvrz.kex.datex2)
 * 
 * Copyright (C) 2014 TraffGo Road GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * TraffGo Road GmbH
 * Ostwall 11-19
 * 47798 Krefeld
 * Phone: +49-2151-3295058
 * mailto: info@traffgoroad.de
 */

package de.bsvrz.kex.datex2.plugins.konvertierung.impl;

import java.util.List;

/**
 * Klasse zur Reprsentation einer ganzen Anzeigetafel.
 * 
 * @author TraffGo Road GmbH
 * @version $Id$
 */

public class AnzeigeTafel {
  /** Id der Anzeigetafel. */
  private String id;
  
  /** Anzeigefeld links. */
  private List<AnzeigeZeile> anzeigenLinks;
  
  /** Anzeigefeld rechts. */
  private List<AnzeigeZeile> anzeigenRechts;
  
  /** Anzeigefeld mitte. */
  private List<AnzeigeZeile> anzeigenMitte;
  
  /** Flag, ob berhaupt ein rechtes Anzeigefeld vorhanden ist.*/
  private boolean hasRechts = false;

  /** Flag, ob berhaupt ein linkes Anzeigefeld vorhanden ist.*/
  private boolean hasLinks = false;

  /** Flag, ob berhaupt ein mittleres Anzeigefeld vorhanden ist.*/
  private boolean hasMitte = false;
	
  /**
   * Konstruktor.
   * 
   * @param id Id der Anzeigetafel
   * @param anzeigenLinks Liste der Zeilen fr linkes Anzeigenfeld oder null, wenn nicht vorhanden
   * @param anzeigenMitte Liste der Zeilen fr mittleres Anzeigenfeld oder null, wenn nicht vorhanden
   * @param anzeigenRechts Liste der Zeilen fr rechtes Anzeigenfeld oder null, wenn nicht vorhanden
   */
	AnzeigeTafel(final String id,  final List<AnzeigeZeile> anzeigenLinks, 
	    final List<AnzeigeZeile> anzeigenMitte, final List<AnzeigeZeile> anzeigenRechts) {
	      this.id = id; 
	      this.anzeigenLinks = anzeigenLinks; 
	      this.anzeigenMitte = anzeigenMitte; 
	      this.anzeigenRechts = anzeigenRechts; 
	      
	      if (this.anzeigenLinks != null) {
	    	  this.hasLinks = true;
	      }
	      
	      if (this.anzeigenRechts != null) {
	    	  this.hasRechts = true;
	      }
	      
	      if (this.anzeigenMitte != null) {
	    	  this.hasMitte = true;
	      }
	    }
	
	/**
	 * Rckgabe der Id der Anzeigentafel.
	 * 
	 * @return String
	 */
	public String getId() {
	    return this.id;
	  }
	
	/**
	 * Rckgabe der Anzeigezeilen des linken Anzeigefeldes.
	 * 
	 * @return List<AnzeigeZeile>
	 */
	
	public List<AnzeigeZeile> getAnzeigenLinks() {
	    return this.anzeigenLinks;
	  }
	

  /**
   * Rckgabe der Anzeigezeilen des rechten Anzeigefeldes.
   * 
   * @return List<AnzeigeZeile>
   */
  
	public List<AnzeigeZeile> getAnzeigenRechts() {
	    return this.anzeigenRechts;
	  }
	

  /**
   * Rckgabe der Anzeigezeilen des mittleren Anzeigefeldes.
   * 
   * @return List<AnzeigeZeile>
   */
  
	public List<AnzeigeZeile> getAnzeigenMitte() {
	    return this.anzeigenMitte;
	  }
	
	/**
	 * Liefert true zurck, wenn berhaupt ein rechtes Anzeigenfeld vorhanden.
	 * 
	 * @return boolean
	 */

  public boolean hasRechts() {
    return this.hasRechts;
  }
  
  /**
   * Liefert true zurck, wenn berhaupt ein linkes Anzeigenfeld vorhanden.
   * 
   * @return boolean
   */

  public boolean hasLinks() {
    return this.hasLinks;
  }
  
  /**
   * Liefert true zurck, wenn berhaupt ein mittleres Anzeigenfeld vorhanden.
   * 
   * @return boolean
   */

  public boolean hasMitte() {
    return this.hasMitte;
  }
	
	
}
