/**
 * SWE KEx-DatexII (de.bsvrz.kex.datex2)
 * 
 * Copyright (C) 2014 TraffGo Road GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * TraffGo Road GmbH
 * Ostwall 11-19
 * 47798 Krefeld
 * Phone: +49-2151-3295058
 * mailto: info@traffgoroad.de
 */

package de.bsvrz.kex.datex2.plugins.konvertierung.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.naming.ConfigurationException;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;

import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.NumberArray;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.kex.datex2.allgemein.ObjekteAtgAsp;
import de.bsvrz.kex.datex2.plugins.definition.PluginExport;
import eu.datex2.schema.statisch.predefined.locations.routen.AffectedCarriagewayAndLanes;
import eu.datex2.schema.statisch.predefined.locations.routen.CarriagewayEnum;
import eu.datex2.schema.statisch.predefined.locations.routen.ConfidentialityValueEnum;
import eu.datex2.schema.statisch.predefined.locations.routen.CountryEnum;
import eu.datex2.schema.statisch.predefined.locations.routen.D2LogicalModel;
import eu.datex2.schema.statisch.predefined.locations.routen.Exchange;
import eu.datex2.schema.statisch.predefined.locations.routen.ExtendedLinear;
import eu.datex2.schema.statisch.predefined.locations.routen.HeaderInformation;
import eu.datex2.schema.statisch.predefined.locations.routen.InformationStatusEnum;
import eu.datex2.schema.statisch.predefined.locations.routen.Intermediate;
import eu.datex2.schema.statisch.predefined.locations.routen.InternationalIdentifier;
import eu.datex2.schema.statisch.predefined.locations.routen.LaneEnum;
import eu.datex2.schema.statisch.predefined.locations.routen.Linear;
import eu.datex2.schema.statisch.predefined.locations.routen.LinearByCoordinates;
import eu.datex2.schema.statisch.predefined.locations.routen.LinearExtensionType;
import eu.datex2.schema.statisch.predefined.locations.routen.PayloadPublication;
import eu.datex2.schema.statisch.predefined.locations.routen.PointCoordinates;
import eu.datex2.schema.statisch.predefined.locations.routen.PredefinedLocation;
import eu.datex2.schema.statisch.predefined.locations.routen.PredefinedLocationsPublication;
import eu.datex2.schema.statisch.predefined.locations.routen.PredefinedNonOrderedLocationGroup;
import eu.datex2.schema.statisch.predefined.locations.routen.SupplementaryPositionalDescription;

/**
 * Konvertierung von statischen Daten (NBA-Routenstcke) in das DATEX-II Format.
 * 
 * @author TraffGo Road GmbH
 * @version $Id$
 */

public class RSPredefinedLocation extends StandardKonvertierungStatisch {
  /** Maximale Anzahl an Empfangversuchen fr Routenlnge. */
  private static final int ANZAHL_VERSUCHE_LAENGENERMITTLUNG = 3;

  /** Zugriff auf RoutenStckLnge. */
  private static final String ROUTEN_STUECK_LAENGE = "RoutenStckLnge";

  /** Schlssel fr maximale Wartezeit. */
  private static final String KEY_MAX_WAIT = "Maximale-Datenwartezeit";
  
  /** Standard-Wert fr maximale Wartezeit. (in Millisekunden) */
  private static final int STANDARD_MAX_WAIT = 1000;
  
  /** Maximale Wartezeit fr Dateninitialisierung. (in Millisekunden) */
  private Integer maxWaitForData = STANDARD_MAX_WAIT;

  /** Routenstck-Lngen. */
  private Map<String, Float> routenLaengen = new HashMap<>();
  
  /**
   * Sehr merkwrdig, dass statische Daten aus dynamischen Daten geholt werden mssen.
   * Wird daher in innere Klasse gekapselt.
   * 
   * @author TraffGo Road GmbH
   * @version $Id$
   */
  
  public class RoutenStueckLaengeErmittlungsKlasse implements ClientReceiverInterface {
    /** Lokaler Cache. */
    private Map<String, ResultData> lokalerCache = new HashMap<>();
    
    /**
     * Konstruktor.
     * 
     * @param ol Liste von Objektliste/Attributgruppe/Aspekt-Kombinationen
     * @throws InterruptedException Unterbrechnung whrend Wartezeit.
     * 
     */
    
    public RoutenStueckLaengeErmittlungsKlasse(final Collection<ObjekteAtgAsp> ol) throws InterruptedException {
      for (ObjekteAtgAsp o : ol) {
        plugin.getDaVConnection().subscribeReceiver(
            this,
            o.getSystemObjekte(),
            o.getDataDescription(),
            ReceiveOptions.delayed(),
            ReceiverRole.receiver(),
            1
            );
      }
      Thread.sleep(maxWaitForData);
      for (ObjekteAtgAsp o : ol) {
        plugin.getDaVConnection().unsubscribeReceiver(this, o.getSystemObjekte(), o.getDataDescription());
      }
    }

      @Override
      public void update(final ResultData[] resultData) {
        for (ResultData x : resultData) {
          lokalerCache.put(x.getObject().getPid(),  x);
        }
      }
    }
     
	  /**
	   * Konstruktor.
	   * 
	   * @param plugin Referenz auf Plugin-Definitions-Klasse
	   */
  
  public RSPredefinedLocation(final PluginExport plugin) {
    this.plugin = plugin;
  }

  /***************************************************************************
   * IN DEM FOLGENDEN ABSCHNITT WERDEN STRUKTURELL GLEICHE METHODEN GESAMMELT.
   * 
   * Diese Methoden werden in den einzelnen Plugins wiederholt - allerdings
   * beziehen sich die JAXB-Elemente trotz gleichen Namens dann auf
   * andere Klassen; eine nominale Gleichheit existiert also nicht.
   ***************************************************************************
   */

  /**
   * Gebe den in dieser Klasse verwendeten JAXBContext zurck. 
   * 
   * @throws JAXBException {@see javax.xml.bind.JAXBContext#newInstance(String)}
   * @return JAXBContext Verwendeter Kontext
   */

  @Override
  public JAXBContext getJAXBContext() throws JAXBException {
    if (jaxbContext == null) {
      jaxbContext = JAXBContext.newInstance(D2LogicalModel.class.getPackage().getName());
    }
    return jaxbContext;
  }

  /**
   * Setzt die allgemeinen Angaben zur Publikation (Exchange, Language, PublicationCreator).
   * 
   * @param d2l D2LogicalModel
   * @param payload Verwendeter Payload
   */

  private void setzeNationalIdentifier(final D2LogicalModel d2l, final PayloadPublication payload) {
    InternationalIdentifier internationalIdentifier = new InternationalIdentifier();
    internationalIdentifier.setCountry(CountryEnum.DE);
    internationalIdentifier.setNationalIdentifier(nationalIdentifier);
    
    // Setze Exchange
    
    Exchange exchange = new Exchange();
    exchange.setSupplierIdentification(internationalIdentifier);
    d2l.setExchange(exchange);

    // Setze Language
    payload.setLang("DE");

    payload.setPublicationCreator(internationalIdentifier);
  }
  
  /***********************************************************************
   * ENDE DES ABSCHNITTS STRUKTURELL GLEICHER METHODEN
   ***********************************************************************
   */

  /** {@inheritDoc} */
  @Override
  public Object getDatexII(final Collection<ObjekteAtgAsp> objekteAtgAspListe, final CreationOptions... options) {
    boolean versionFree = WerkzeugSammlung.isVersionFree(options);
    String newVersion = WerkzeugSammlung.erzeugeVersionsString(versionFree);

    D2LogicalModel d2l = new D2LogicalModel();
    PredefinedLocationsPublication payload = new PredefinedLocationsPublication();
    d2l.setPayloadPublication(payload);

    d2l.setExtensionName(d2l.getExtensionName());
    d2l.setExtensionVersion(d2l.getExtensionVersion());
    
    setzeNationalIdentifier(d2l, payload);

    HeaderInformation headerInformation = new HeaderInformation();
    headerInformation.setConfidentiality(ConfidentialityValueEnum.NO_RESTRICTION);
    headerInformation.setInformationStatus(InformationStatusEnum.REAL);
    payload.setHeaderInformation(headerInformation);
    
    payload.setPublicationTime(WerkzeugSammlung.erzeugeVersionsdatum(versionFree));
    
    PredefinedNonOrderedLocationGroup predefinedNonOrderedLocationGroup = new PredefinedNonOrderedLocationGroup();
    predefinedNonOrderedLocationGroup.setId(locationGroupId);
    predefinedNonOrderedLocationGroup.setVersion(newVersion);

    for (ObjekteAtgAsp objekteAtgAsp : objekteAtgAspListe) {
      for (SystemObject mq : objekteAtgAsp.getSystemObjekte()) {
        try {
          AttributeGroup attributGruppe = mq.getDataModel().getAttributeGroup("atg.nbaRoutenStck");
          Data dataRoutenStueck = mq.getConfigurationData(attributGruppe);

          if (dataRoutenStueck != null) {  
            List <Data> dataKoordinaten = new ArrayList<Data>();
            SystemObject routenStueck = dataRoutenStueck.getReferenceValue("RoutenStckReferenz").getSystemObject();
            Data koordData = getAtgKoordinaten(routenStueck);
            if (koordData == null) {
              dataKoordinaten = getLinienReferenz(dataKoordinaten, routenStueck);
            } else {
              dataKoordinaten.add(koordData);
            }	        	
            List<PointCoordinates> koordinatenListe = getKoordinatenListe(dataKoordinaten);
            LinearByCoordinates linearByCoordinates = getLinearByCoordinates(koordinatenListe);
            ExtendedLinear extendedLinear = new ExtendedLinear();
            extendedLinear.setLinearByCoordinates(linearByCoordinates);
            LinearExtensionType linearExtensionType = new LinearExtensionType();
            linearExtensionType.setExtendedLinear(extendedLinear);
            AffectedCarriagewayAndLanes affectedCarriagewayAndLanes = new AffectedCarriagewayAndLanes();
            affectedCarriagewayAndLanes.setCarriageway(CarriagewayEnum.MAIN_CARRIAGEWAY);
            affectedCarriagewayAndLanes.getLanes().add(LaneEnum.ALL_LANES_COMPLETE_CARRIAGEWAY);
            affectedCarriagewayAndLanes.setLengthAffected(routenLaengen.get(mq.getPid()));
            SupplementaryPositionalDescription supplementaryPositionalDescription = 
                new SupplementaryPositionalDescription();
            supplementaryPositionalDescription.getAffectedCarriagewayAndLanes().add(affectedCarriagewayAndLanes);
            Linear linear = new Linear();
            linear.setLinearExtension(linearExtensionType);
            linear.setSupplementaryPositionalDescription(supplementaryPositionalDescription);

            PredefinedLocation predefinedLocation = new PredefinedLocation();
            predefinedLocation.setVersion(plugin.getPredefinedLocationVersion());
            String pid = Long.valueOf(mq.getId()).toString();
            predefinedLocation.setId(pid);
            predefinedLocation.setVersion(newVersion);
            predefinedLocation.setLocation(linear);
            predefinedNonOrderedLocationGroup.getPredefinedLocations().add(predefinedLocation);
          } // (d != null)
        } catch (Exception e) {
          LOGGER.info(logPrefix + "berspringe statisches Datum.", e);
        }
      }
    }
    
    if (predefinedNonOrderedLocationGroup.getPredefinedLocations().size() < 2) {
      return null;
    } else {
      if (!versionFree) {
        this.version = newVersion;
        try {
          plugin.getPersistenzModul().putData(KEY_PREDEFINED_LOCATION_VERSION, version);
        } catch (Exception e) {
          LOGGER.warning(logPrefix + "PredefinedLocationVersion kann nicht persistiert werden.", e);
        }
      }
      payload.getPredefinedLocationContainers().add(predefinedNonOrderedLocationGroup);
      return d2l;
    }
  }
  
 
  /**
   * Holt rekursiv LineienReferenzObjekte aus den Konfigurationsdaten, bis Koordinaten gefunden werden.
   * 
   * @param dataKoordinaten
   *          Data
   * @param systemObject
   *          SystemObject
   * @return List<Data>
   */
  
  protected List<Data> getLinienReferenz(final List<Data> dataKoordinaten, final SystemObject systemObject) {
    List<Data> resultDataKoordinaten = dataKoordinaten;
    AttributeGroup attGruppe = systemObject.getDataModel().getAttributeGroup("atg.bestehtAusLinienObjekten");
 	  Data dataLinien = systemObject.getConfigurationData(attGruppe);
 	  if (dataLinien != null) {
 		 SystemObject[] sysObj = dataLinien.getReferenceArray("LinienReferenz").getSystemObjectArray();
 		 for (int i = 0; i < sysObj.length; i++)	{
 			 Data d = getAtgKoordinaten(sysObj[i]);
 			 if (d == null) {
 				resultDataKoordinaten = getLinienReferenz(dataKoordinaten, sysObj[i]);     
 			 } else {
 				resultDataKoordinaten.add(d);
 			 }		
 		 } // end for
 	 } //linien != null
 	  return resultDataKoordinaten;
  }
 
  /**
   * Holt LinienKoordinaten aus den Konfigurationsdaten von SystemObject.
   * 
   * @param systemObject
   *          SystemObject
   * @return Data
   */
  
  protected Data getAtgKoordinaten(final SystemObject systemObject) {
	  AttributeGroup attGruppe = systemObject.getDataModel().getAttributeGroup("atg.linienKoordinaten");
	 	Data d = systemObject.getConfigurationData(attGruppe);
	 	return d;
  }
  
  
  /**
   * Baut eine Liste von PointCoordinates aus einer Data-Liste.
   * 
   * @param dataList
   *          List<Data>
   * @return List<PointCoordinates>
   */
  
  protected List<PointCoordinates> getKoordinatenListe(final List<Data> dataList) {
	 List<PointCoordinates> pointCoordinatesList = new ArrayList<PointCoordinates>();
	 for (Data data : dataList)  {
		 NumberArray xArray = data.getScaledArray("x");
		 NumberArray yArray = data.getScaledArray("y");
		 for (int i = 0; i < xArray.getLength(); i++) {
			 PointCoordinates pointCoordinates = new PointCoordinates();
			 pointCoordinates.setLatitude(xArray.floatValue(i));
			 pointCoordinates.setLongitude(yArray.floatValue(i));
			 pointCoordinatesList.add(pointCoordinates);
		    }
	 }
	 return pointCoordinatesList;
  }
  
  
  /**
   * Baut eine LinearByCoordinates-Objekt aus PointCoordinates-Liste.
   * 
   * @param koordinatenListe
   *          List<PointCoordinates>
   * @return LinearByCoordinates
   */

  protected LinearByCoordinates getLinearByCoordinates(final List<PointCoordinates> koordinatenListe) {
    LinearByCoordinates linearByCoordinates = new LinearByCoordinates();
    int ende = koordinatenListe.size() - 1;
    int index = 0;
    for (int i = 0; i < koordinatenListe.size(); i++) {
      if (i == 0) {
        linearByCoordinates.setStart(koordinatenListe.get(i));
      } else if (i == ende) {
        linearByCoordinates.setEnd(koordinatenListe.get(i));
      } else {
        Intermediate intermediate = new Intermediate();
        intermediate.setIndex(index);
        index++;
        intermediate.setPointCoordinates(koordinatenListe.get(i));
        linearByCoordinates.getIntermediates().add(intermediate);
      }
    }
	  return linearByCoordinates;
  }
  
  @Override
  public void initialisiere(final Collection<ObjekteAtgAsp> objects) throws ConfigurationException {
    initialisiereBasisFelder();
    
    if (!plugin.getKonfigurationsModul().hasData(KEY_MAX_WAIT)) {
      LOGGER.info(logPrefix + "Das Feld '" + KEY_MAX_WAIT 
          + "' wurde nicht in der Konfiguration angegeben, nutze Standard-Wert:" + maxWaitForData);
    } else {
      maxWaitForData = plugin.getKonfigurationsModul().getInteger(KEY_MAX_WAIT);
    }

    String errPid = "";
    try {
      int attempt;
      for (attempt = ANZAHL_VERSUCHE_LAENGENERMITTLUNG; attempt >= 0; attempt--) {
        RoutenStueckLaengeErmittlungsKlasse rslek = new RoutenStueckLaengeErmittlungsKlasse(objects);
        for (ResultData r : rslek.lokalerCache.values()) {
          String pid = r.getObject().getPid();
          errPid = pid;
          Float length;
          try {
            length = r.getData().getItem(ROUTEN_STUECK_LAENGE).asScaledValue().floatValue();
          } catch (Exception ex) {
            // Falls irgendwas an diesem Verfahren schief geht, wird 0 als Lnge gesetzt.
            LOGGER.fine(logPrefix + "Routenstck-Bestimmung fehlgeschlagen.", ex);
            length = 0f;
          }
          if (!routenLaengen.containsKey(pid)) {
            if ((attempt == 0) || (length > 0)) {
              routenLaengen.put(pid, length);
              if (length.equals(0f)) {
                LOGGER.warning(logPrefix + "Routenstck-Bestimmung fr " + pid 
                    + " fehlgeschlagen, nehme 0m an.");                
              }
            }  
          }
        }
      }
    } catch (Exception e) {
      LOGGER.error(logPrefix + "Ermittlung der Routenlnge konnte nicht durchgefhrt werden (" + errPid + ").", e);
    }    
  }
}
