/**
 * SWE KEx-DatexII (de.bsvrz.kex.datex2)
 * 
 * Copyright (C) 2014 TraffGo Road GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * TraffGo Road GmbH
 * Ostwall 11-19
 * 47798 Krefeld
 * Phone: +49-2151-3295058
 * mailto: info@traffgoroad.de
 */

package de.bsvrz.kex.datex2.plugins.konvertierung.impl;

import java.math.BigInteger;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.Map;

import javax.naming.ConfigurationException;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.datatype.XMLGregorianCalendar;

import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.kex.datex2.allgemein.ObjekteAtgAsp;
import de.bsvrz.kex.datex2.plugins.definition.PluginExport;
import eu.datex2.schema.dynamisch.segmente.verkehrslage.PayloadPublication;
import eu.datex2.schema.dynamisch.segmente.verkehrslage.ConcentrationOfVehiclesValue;
import eu.datex2.schema.dynamisch.segmente.verkehrslage.ConfidentialityValueEnum;
import eu.datex2.schema.dynamisch.segmente.verkehrslage.CountryEnum;
import eu.datex2.schema.dynamisch.segmente.verkehrslage.D2LogicalModel;
import eu.datex2.schema.dynamisch.segmente.verkehrslage.DurationValue;
import eu.datex2.schema.dynamisch.segmente.verkehrslage.ElaboratedData;
import eu.datex2.schema.dynamisch.segmente.verkehrslage.ElaboratedDataPublication;
import eu.datex2.schema.dynamisch.segmente.verkehrslage.Exchange;
import eu.datex2.schema.dynamisch.segmente.verkehrslage.HeaderInformation;
import eu.datex2.schema.dynamisch.segmente.verkehrslage.InformationStatusEnum;
import eu.datex2.schema.dynamisch.segmente.verkehrslage.InternationalIdentifier;
import eu.datex2.schema.dynamisch.segmente.verkehrslage.LocationByReference;
import eu.datex2.schema.dynamisch.segmente.verkehrslage.MultilingualString;
import eu.datex2.schema.dynamisch.segmente.verkehrslage.MultilingualString.Values;
import eu.datex2.schema.dynamisch.segmente.verkehrslage.MultilingualStringValue;
import eu.datex2.schema.dynamisch.segmente.verkehrslage.OverallPeriod;
import eu.datex2.schema.dynamisch.segmente.verkehrslage.PredefinedLocationVersionedReference;
import eu.datex2.schema.dynamisch.segmente.verkehrslage.PredefinedNonOrderedLocationGroupVersionedReference;
import eu.datex2.schema.dynamisch.segmente.verkehrslage.ReferenceSettings;
import eu.datex2.schema.dynamisch.segmente.verkehrslage.SpeedValue;
import eu.datex2.schema.dynamisch.segmente.verkehrslage.TrafficConcentration;
import eu.datex2.schema.dynamisch.segmente.verkehrslage.TrafficFlow;
import eu.datex2.schema.dynamisch.segmente.verkehrslage.TrafficSpeed;
import eu.datex2.schema.dynamisch.segmente.verkehrslage.TrafficStatus;
import eu.datex2.schema.dynamisch.segmente.verkehrslage.TrafficStatusEnum;
import eu.datex2.schema.dynamisch.segmente.verkehrslage.TrafficStatusValue;
import eu.datex2.schema.dynamisch.segmente.verkehrslage.TravelTimeData;
import eu.datex2.schema.dynamisch.segmente.verkehrslage.Validity;
import eu.datex2.schema.dynamisch.segmente.verkehrslage.ValidityStatusEnum;
import eu.datex2.schema.dynamisch.segmente.verkehrslage.VehicleCharacteristics;
import eu.datex2.schema.dynamisch.segmente.verkehrslage.VehicleFlowValue;
import eu.datex2.schema.dynamisch.segmente.verkehrslage.VehicleTypeEnum;

/**
 * Konvertierung der Verkehrslage aus OLSIM in das DATEX-II Format.
 * 
 * @author TraffGo Road GmbH
 * @version $Id$
 */

public class SDVerkehrslage extends StandardKonvertierungDynamisch {
  /** Faktor zur Umrechnung [0;1] => Prozent [0;100]. */
  private static final int FACTOR_PERCENT = 100;
  
  /**
   * Konstruktor.
   * 
   * @param plugin Referenz auf Plugin-Definitions-Klasse.
   */
  
  public SDVerkehrslage(final PluginExport plugin) {
    this.plugin = plugin;
  }

  /***************************************************************************
   * IN DEM FOLGENDEN ABSCHNITT WERDEN STRUKTURELL GLEICHE METHODEN GESAMMELT.
   * 
   * Diese Methoden werden in den einzelnen Plugins wiederholt - allerdings
   * beziehen sich die JAXB-Elemente trotz gleichen Namens dann auf
   * andere Klassen; eine nominale Gleichheit existiert also nicht.
   ***************************************************************************
   */

  /**
   * Gebe den in dieser Klasse verwendeten JAXBContext zurck. 
   * 
   * @throws JAXBException {@see javax.xml.bind.JAXBContext#newInstance(String)}
   * @return JAXBContext Verwendeter Kontext
   */

  @Override
  public JAXBContext getJAXBContext() throws JAXBException {
    if (jaxbContext == null) {
      jaxbContext = JAXBContext.newInstance(D2LogicalModel.class.getPackage().getName());
    }
    return jaxbContext;
  }

  /**
   * Setzt die allgemeinen Angaben zur Publikation (Exchange, Language, PublicationCreator).
   * 
   * @param d2l D2LogicalModel
   * @param payload Payload des D2LogicalModel
   */

  private void setzeNationalIdentifier(final D2LogicalModel d2l, final PayloadPublication payload) {
    InternationalIdentifier internationalIdentifier = new InternationalIdentifier();
    internationalIdentifier.setCountry(CountryEnum.DE);
    internationalIdentifier.setNationalIdentifier(nationalIdentifier);
    
    // Setze Exchange
    
    Exchange exchange = new Exchange();
    exchange.setSupplierIdentification(internationalIdentifier);
    d2l.setExchange(exchange);

    // Setze Language
    payload.setLang("DE");

    payload.setPublicationCreator(internationalIdentifier);
  }
  
  /***********************************************************************
   * ENDE DES ABSCHNITTS STRUKTURELL GLEICHER METHODEN
   ***********************************************************************
   */

  /** {@inheritDoc} */
  @Override
  public Object getDatexII(final Map<ObjekteAtgAsp, Collection<ResultData>> data) {
    String predefinedLocationVersion = holePredefinedLocationVersion();

    D2LogicalModel d2l = new D2LogicalModel();
    ElaboratedDataPublication payload = new ElaboratedDataPublication();

    setzeNationalIdentifier(d2l, payload);
    
   HeaderInformation headerInformation = new HeaderInformation();
   headerInformation.setConfidentiality(ConfidentialityValueEnum.RESTRICTED_TO_AUTHORITIES);
   headerInformation.setInformationStatus(InformationStatusEnum.REAL);
   payload.setHeaderInformation(headerInformation);
 
   ReferenceSettings referenceSettings = new ReferenceSettings();
   PredefinedNonOrderedLocationGroupVersionedReference predefinedNonOrderedLocationGroupVersionedReference =
       new PredefinedNonOrderedLocationGroupVersionedReference();
   predefinedNonOrderedLocationGroupVersionedReference.setId(locationGroupId);
   predefinedNonOrderedLocationGroupVersionedReference.setVersion(predefinedLocationVersion);
   referenceSettings.setPredefinedNonOrderedLocationGroupReference(predefinedNonOrderedLocationGroupVersionedReference);
   referenceSettings.setTrafficStatusDefault(TrafficStatusEnum.FREE_FLOW);
   payload.setReferenceSettings(referenceSettings);
   
   Date datum = new Date();
   XMLGregorianCalendar xmlGregorianCalendar = WerkzeugSammlung.konvertiereDatum(datum);
   payload.setPublicationTime(xmlGregorianCalendar);
   payload.setPeriodDefault(intervall);
   
    for (ObjekteAtgAsp objekteAtgAsp : data.keySet()) {
    	for (ResultData resultData : data.get(objekteAtgAsp)) {	
	         if (resultData != null) { 
	        	try {
		            LocationByReference locationByReference = new LocationByReference();
		            PredefinedLocationVersionedReference predefinedLocationVersionedReference = 
		                new PredefinedLocationVersionedReference();
		            predefinedLocationVersionedReference.setId(resultData.getObject().getPid());
		            predefinedLocationVersionedReference.setVersion(predefinedLocationVersion);
		            locationByReference.setPredefinedLocationReference(predefinedLocationVersionedReference);
		            addTrafficStatus(payload, resultData, locationByReference);
		            addTrafficData(payload, resultData, locationByReference);
		            addTravelTime(payload, resultData, locationByReference);
	        	} catch (Exception e) {
	                 LOGGER.error(logPrefix + "Daten nicht bercksichtigt.", e);
	             }
	          } //if(resultData != null)	        
        } // end for 
    } // end for
    d2l.setPayloadPublication(payload);

    if (payload.getElaboratedDatas().isEmpty()) {
      LOGGER.warning(logPrefix + "Kein valider Datensatz verfgbar, Publikation wird abgebrochen");
      return null;
    } else { 
      return d2l;
    }
  }
  
  /**
   * Setzt den TrafficStatus aus Resultdata und erweitert die ElaboratedDataPublication.
   * 
   * @param payload
   *          ElaboratedDataPublication
   * @param resultData 
   * 			ResultData 
   * @param locationByReference 
   * 			LocationByReference 
   */
  
  protected void addTrafficStatus(final ElaboratedDataPublication payload, 
      final ResultData resultData, final LocationByReference locationByReference) {
    ElaboratedData elaboratedData = new ElaboratedData();
    TrafficStatus trafficStatus = new TrafficStatus();
    if (resultData.hasData()) {
      boolean hasElaboratedData = false;
      long referenzZeit = resultData.getData().getTimeValue("ReferenzZeit").getMillis();
      XMLGregorianCalendar xmlReferenzZeit = WerkzeugSammlung.konvertiereDatum(referenzZeit);

      trafficStatus.setMeasurementOrCalculationTime(xmlReferenzZeit);
      String situation = resultData.getData().getTextValue("Situation").getValueText();
      long horizont =  resultData.getData().getTimeValue("PrognoseHorizont").getMillis();

      float gueteIndex = WerkzeugSammlung.getFloatValue(resultData, "Gte", "Index", SMALL_MINUS_FLOAT, 2);
      int horizontMilliSekunden = WerkzeugSammlung.getIntValue(horizont);

      Float gueteProzent = null;
      if (gueteIndex >= 0) {
        gueteProzent = Float.valueOf((1.0f - gueteIndex) * FACTOR_PERCENT);
      }

      trafficStatus.setPertinentLocation(locationByReference);

      TrafficStatusValue trafficStatusValue = new TrafficStatusValue();
      trafficStatus.setTrafficStatus(trafficStatusValue);

      trafficStatusValue.setAccuracy(gueteProzent);

      switch (situation.trim().toLowerCase()) {
      case "strung":
        trafficStatusValue.setDataError(true);
        trafficStatusValue.setReasonForDataError(konvertiereString("Strung"));
        trafficStatusValue.setTrafficStatusValue(TrafficStatusEnum.UNKNOWN);
        hasElaboratedData = false; // true
        break;
      case "keine aussage":
        trafficStatusValue.setTrafficStatusValue(TrafficStatusEnum.UNKNOWN);
        hasElaboratedData = true;
        break;
      case "lebhafter verkehr":
      case "dichter verkehr":
        trafficStatusValue.setTrafficStatusValue(TrafficStatusEnum.HEAVY);
        hasElaboratedData = true;
        break;
      case "zhflieender verkehr":
      case "stockender verkehr":
        trafficStatusValue.setTrafficStatusValue(TrafficStatusEnum.CONGESTED);
        hasElaboratedData = true;
        break;
      case "stau":
        trafficStatusValue.setTrafficStatusValue(TrafficStatusEnum.IMPOSSIBLE);
        hasElaboratedData = true;
        break;
      case "freier verkehr":
        trafficStatusValue.setTrafficStatusValue(TrafficStatusEnum.FREE_FLOW);
        hasElaboratedData = false;
        break;
      default:
        //Defaultwert Freier Verkehr = TrafficStatusEnum.FREE_FLOW
        trafficStatusValue.setTrafficStatusValue(TrafficStatusEnum.UNKNOWN);
        trafficStatusValue.setDataError(true);
        trafficStatusValue.setReasonForDataError(konvertiereString("unbekannt"));
        hasElaboratedData = false;  // true
        break;
      }
      if (hasElaboratedData) {
        elaboratedData.setBasicData(trafficStatus);
        if (horizont > 0) { 
          elaboratedData.setValidity(setzeValidity(horizontMilliSekunden));
          elaboratedData.setForecast(true);
        }
        payload.getElaboratedDatas().add(elaboratedData);
      } // hasElaboratedData
    } 
  }
  
  /**
   * Setzt TrafficData aus Resultdata und erweitert die ElaboratedDataPublication.
   * 
   * @param payload
   *          ElaboratedDataPublication
   * @param resultData 
   * 			ResultData 
   * @param locationByReference 
   * 			LocationByReference 
   */
  
  protected void addTrafficData(final ElaboratedDataPublication payload, 
      final ResultData resultData, final LocationByReference locationByReference) {
	 
	  if (resultData.hasData()) {
	      long referenzZeit = resultData.getData().getTimeValue("ReferenzZeit").getMillis();
	      XMLGregorianCalendar xmlReferenzZeit = WerkzeugSammlung.konvertiereDatum(referenzZeit);
	    	 
		  VehicleCharacteristics vehicleCharacteristicsPkw = new VehicleCharacteristics();
	      vehicleCharacteristicsPkw.getVehicleTypes().add(VehicleTypeEnum.CAR);
	      VehicleCharacteristics vehicleCharacteristicsLkw = new VehicleCharacteristics();
	      vehicleCharacteristicsLkw.getVehicleTypes().add(VehicleTypeEnum.LORRY);
	      VehicleCharacteristics vehicleCharacteristicsKfz = new VehicleCharacteristics();
	      vehicleCharacteristicsKfz.getVehicleTypes().add(VehicleTypeEnum.ANY_VEHICLE);
	
	      //FlussPkw      
	      long flussPkw = WerkzeugSammlung.getLongValue(resultData, "FlussPkw", SMALL_MINUS_LONG);
	      ElaboratedData elaboratedDataFlussPkw = new ElaboratedData();
	      TrafficFlow trafficFlowPkw = new TrafficFlow();
	      trafficFlowPkw.setForVehiclesWithCharacteristicsOf(vehicleCharacteristicsPkw);
	      trafficFlowPkw.setMeasurementOrCalculationTime(payload.getPublicationTime());
	      trafficFlowPkw.setPertinentLocation(locationByReference);
	      VehicleFlowValue vehicleFlowValuePkw = new VehicleFlowValue();
	      if (flussPkw >= 0) {
	        vehicleFlowValuePkw.setVehicleFlowRate(BigInteger.valueOf(flussPkw));
	        trafficFlowPkw.setVehicleFlow(vehicleFlowValuePkw);
	        elaboratedDataFlussPkw.setBasicData(trafficFlowPkw);
	        payload.getElaboratedDatas().add(elaboratedDataFlussPkw);
	      } 
	
	      //FlussLkw
	      long flussLkw = WerkzeugSammlung.getLongValue(resultData, "FlussLkw", SMALL_MINUS_LONG);
	      ElaboratedData elaboratedDataFlussLkw = new ElaboratedData();
	      TrafficFlow trafficFlowLkw = new TrafficFlow(); 
	      trafficFlowLkw.setForVehiclesWithCharacteristicsOf(vehicleCharacteristicsLkw);
	      trafficFlowLkw.setMeasurementOrCalculationTime(xmlReferenzZeit);
	      trafficFlowLkw.setPertinentLocation(locationByReference);
	      VehicleFlowValue vehicleFlowValueLkw = new VehicleFlowValue();
	      if (flussLkw >= 0) {
	        vehicleFlowValueLkw.setVehicleFlowRate(BigInteger.valueOf(flussLkw));
	        trafficFlowLkw.setVehicleFlow(vehicleFlowValueLkw);
	        elaboratedDataFlussLkw.setBasicData(trafficFlowLkw);
	        payload.getElaboratedDatas().add(elaboratedDataFlussLkw);
	      } 
	
	      //FlussKfz
	      long flussKfz = WerkzeugSammlung.getLongValue(resultData, "FlussKfz",  SMALL_MINUS_LONG);

	      ElaboratedData elaboratedDataFlussKfz = new ElaboratedData();
	      TrafficFlow trafficFlowKfz = new TrafficFlow();
	      trafficFlowKfz.setForVehiclesWithCharacteristicsOf(vehicleCharacteristicsKfz);
	      trafficFlowKfz.setMeasurementOrCalculationTime(xmlReferenzZeit);
	      trafficFlowKfz.setPertinentLocation(locationByReference);
	      VehicleFlowValue vehicleFlowValueKfz = new VehicleFlowValue();
	     
	      if (flussKfz >= 0) {
	        vehicleFlowValueKfz.setVehicleFlowRate(BigInteger.valueOf(flussKfz));
	        trafficFlowKfz.setVehicleFlow(vehicleFlowValueKfz);
	        elaboratedDataFlussKfz.setBasicData(trafficFlowKfz); 
	        payload.getElaboratedDatas().add(elaboratedDataFlussKfz);
	      } 
	  	
	      //VPkw      
	      long vPkw = WerkzeugSammlung.getLongValue(resultData, "vPkw", SMALL_MINUS_LONG);
	      ElaboratedData elaboratedDataVPkw = new ElaboratedData();
	      TrafficSpeed trafficSpeedPkw = new TrafficSpeed();
	      trafficSpeedPkw.setForVehiclesWithCharacteristicsOf(vehicleCharacteristicsPkw);
	      trafficSpeedPkw.setMeasurementOrCalculationTime(xmlReferenzZeit);
	      trafficSpeedPkw.setPertinentLocation(locationByReference);
	      SpeedValue speedValuePkw = new SpeedValue();
	      if (vPkw >= 0) {
	        speedValuePkw.setSpeed(vPkw);
	        trafficSpeedPkw.setAverageVehicleSpeed(speedValuePkw);
	        elaboratedDataVPkw.setBasicData(trafficSpeedPkw);
	        payload.getElaboratedDatas().add(elaboratedDataVPkw);
	      } 
	
	      //VLkw      
	      long vLkw = WerkzeugSammlung.getLongValue(resultData, "vLkw", SMALL_MINUS_LONG);
	      ElaboratedData elaboratedDataVLkw = new ElaboratedData();
	      TrafficSpeed trafficSpeedLkw = new  TrafficSpeed();
	      trafficSpeedLkw.setForVehiclesWithCharacteristicsOf(vehicleCharacteristicsLkw);
	      trafficSpeedLkw.setMeasurementOrCalculationTime(xmlReferenzZeit);
	      trafficSpeedLkw.setPertinentLocation(locationByReference);
	      SpeedValue speedValueLkw = new SpeedValue();
	      if (vLkw >= 0) {
	        speedValueLkw.setSpeed(vLkw);
	        trafficSpeedLkw.setAverageVehicleSpeed(speedValueLkw);
	        elaboratedDataVLkw.setBasicData(trafficSpeedLkw);
	        payload.getElaboratedDatas().add(elaboratedDataVLkw);
	      } 
	
	      //Vkfz      
	      long vKfz = WerkzeugSammlung.getLongValue(resultData, "vKfz", SMALL_MINUS_LONG);
	      ElaboratedData elaboratedDataVKfz = new ElaboratedData();
	      TrafficSpeed trafficSpeedKfz = new TrafficSpeed();
	      trafficSpeedKfz.setForVehiclesWithCharacteristicsOf(vehicleCharacteristicsKfz);
	      trafficSpeedKfz.setMeasurementOrCalculationTime(xmlReferenzZeit);
	      trafficSpeedKfz.setPertinentLocation(locationByReference);
	      SpeedValue speedValueKfz = new SpeedValue();
	      if (vKfz >= 0) {
	        speedValueKfz.setSpeed(vKfz);
	        trafficSpeedKfz.setAverageVehicleSpeed(speedValueKfz);
	        elaboratedDataVKfz.setBasicData(trafficSpeedKfz);
	        payload.getElaboratedDatas().add(elaboratedDataVKfz);   
	      } 
	      
	      //DichtePkw     
	      long dichtePkw = WerkzeugSammlung.getLongValue(resultData, "DichtePkw", SMALL_MINUS_LONG);
	      ElaboratedData elaboratedDataDichtePkw = new ElaboratedData();
	      TrafficConcentration trafficConcentrationPkw = new TrafficConcentration();
	      trafficConcentrationPkw.setForVehiclesWithCharacteristicsOf(vehicleCharacteristicsPkw);
	      trafficConcentrationPkw.setMeasurementOrCalculationTime(xmlReferenzZeit);
	      trafficConcentrationPkw.setPertinentLocation(locationByReference);
	      ConcentrationOfVehiclesValue concentrationOfVehiclesPkw = new ConcentrationOfVehiclesValue();
	      if (dichtePkw >= 0) {
	    	  concentrationOfVehiclesPkw.setConcentrationOfVehicles(BigInteger.valueOf(dichtePkw));
	        trafficConcentrationPkw.setConcentration(concentrationOfVehiclesPkw);
	        elaboratedDataDichtePkw.setBasicData(trafficConcentrationPkw);
	        payload.getElaboratedDatas().add(elaboratedDataDichtePkw);
	      }
	      
	      //DichteLkw     
	      long dichteLkw = WerkzeugSammlung.getLongValue(resultData, "DichteLkw", SMALL_MINUS_LONG);
	      ElaboratedData elaboratedDataDichteLkw = new ElaboratedData();
	      TrafficConcentration trafficConcentrationLkw = new TrafficConcentration();
	      trafficConcentrationLkw.setForVehiclesWithCharacteristicsOf(vehicleCharacteristicsLkw);
	      trafficConcentrationLkw.setMeasurementOrCalculationTime(xmlReferenzZeit);
	      trafficConcentrationLkw.setPertinentLocation(locationByReference);
	      ConcentrationOfVehiclesValue concentrationOfVehiclesLkw = new ConcentrationOfVehiclesValue();
	      if (dichteLkw >= 0) {
	    	  concentrationOfVehiclesLkw.setConcentrationOfVehicles(BigInteger.valueOf(dichteLkw));
	        trafficConcentrationLkw.setConcentration(concentrationOfVehiclesLkw);
	        elaboratedDataDichteLkw.setBasicData(trafficConcentrationLkw);
	        payload.getElaboratedDatas().add(elaboratedDataDichteLkw);
	      } 
	      
	    //DichteLkw     
	      long dichteKfz = WerkzeugSammlung.getLongValue(resultData, "DichteKfz", SMALL_MINUS_LONG);
	      ElaboratedData elaboratedDataDichteKfz = new ElaboratedData();
	      TrafficConcentration trafficConcentrationKfz = new TrafficConcentration();
	      trafficConcentrationKfz.setForVehiclesWithCharacteristicsOf(vehicleCharacteristicsKfz);
	      trafficConcentrationKfz.setMeasurementOrCalculationTime(xmlReferenzZeit);
	      trafficConcentrationKfz.setPertinentLocation(locationByReference);
	      ConcentrationOfVehiclesValue concentrationOfVehiclesKfz = new ConcentrationOfVehiclesValue();
	      if (dichteKfz >= 0) {
	    	  concentrationOfVehiclesKfz.setConcentrationOfVehicles(BigInteger.valueOf(dichteKfz));
	        trafficConcentrationKfz.setConcentration(concentrationOfVehiclesKfz);
	        elaboratedDataDichteKfz.setBasicData(trafficConcentrationKfz);
	        payload.getElaboratedDatas().add(elaboratedDataDichteKfz);
	      }
	      
	  }
  }
  
  /**
   * setzt die Reisezeit aus Resultdata und erweitert die ElaboratedDataPublication.
   * 
   * @param payload
   *          ElaboratedDataPublication
   * @param resultData 
   * 			ResultData 
   * @param locationByReference 
   * 			LocationByReference 
   */
  
  protected void addTravelTime(final ElaboratedDataPublication payload, 
      final ResultData resultData, final LocationByReference locationByReference) {
	  
	  if (resultData.hasData()) {
	      long referenzZeit = resultData.getData().getTimeValue("ReferenzZeit").getMillis();
	      XMLGregorianCalendar xmlReferenzZeit = WerkzeugSammlung.konvertiereDatum(referenzZeit);

	      //ReisezeitPkw
	      long reisezeitPkw =  resultData.getData().getTimeValue("ReisezeitPkw").getSeconds();
	      ElaboratedData elaboratedDataZeitPkw = new ElaboratedData();
	      TravelTimeData travelTimeDataPkw = new TravelTimeData();
	      DurationValue durationValuePkw = new DurationValue();
	      travelTimeDataPkw.getVehicleTypes().add(VehicleTypeEnum.CAR);
	      travelTimeDataPkw.setMeasurementOrCalculationTime(xmlReferenzZeit);
	      travelTimeDataPkw.setPertinentLocation(locationByReference);
	      if (reisezeitPkw > 0) {
	    	  durationValuePkw.setDuration(reisezeitPkw);
	        travelTimeDataPkw.setTravelTime(durationValuePkw);
	        elaboratedDataZeitPkw.setBasicData(travelTimeDataPkw);
	        payload.getElaboratedDatas().add(elaboratedDataZeitPkw);
	      } 
	      
	      
	      
	    //ReisezeitPkw
	      long reisezeitLkw =  resultData.getData().getTimeValue("ReisezeitLkw").getSeconds();
	      ElaboratedData elaboratedDataZeitLkw = new ElaboratedData();
	      TravelTimeData travelTimeDataLkw = new TravelTimeData();
	      DurationValue durationValueLkw = new DurationValue();
	      travelTimeDataLkw.getVehicleTypes().add(VehicleTypeEnum.LORRY);
	      travelTimeDataLkw.setMeasurementOrCalculationTime(xmlReferenzZeit);
	      travelTimeDataLkw.setPertinentLocation(locationByReference);
	      if (reisezeitLkw > 0) {
	    	  durationValueLkw.setDuration(reisezeitLkw);
	        travelTimeDataLkw.setTravelTime(durationValueLkw);
	        elaboratedDataZeitLkw.setBasicData(travelTimeDataLkw);
	        payload.getElaboratedDatas().add(elaboratedDataZeitLkw);
	      }
	      
	      //ReisezeitKfz
	      long reisezeitKfz =  resultData.getData().getTimeValue("ReisezeitKfz").getSeconds();
	      ElaboratedData elaboratedDataZeitKfz = new ElaboratedData();
	      TravelTimeData travelTimeDataKfz = new TravelTimeData();
	      DurationValue durationValueKfz = new DurationValue();
	      travelTimeDataKfz.getVehicleTypes().add(VehicleTypeEnum.ANY_VEHICLE);
	      travelTimeDataKfz.setMeasurementOrCalculationTime(xmlReferenzZeit);
	      travelTimeDataKfz.setPertinentLocation(locationByReference);
	      if (reisezeitKfz > 0) {
	    	  durationValueKfz.setDuration(reisezeitKfz);
	        travelTimeDataKfz.setTravelTime(durationValueKfz);
	        elaboratedDataZeitKfz.setBasicData(travelTimeDataKfz);
	        payload.getElaboratedDatas().add(elaboratedDataZeitKfz);
	      } 
	  }
  }
  
  
  /**
   * setzt ein Start und Enddatum in ein DatexII Validity-Objekt. 
   * 
   * @param horizont
   *          Integer Millisekunden als Intervalldauer
   * @return Validity
   */

  protected Validity setzeValidity(final int horizont) {
	  
	  Validity validity = new Validity();
	  validity.setValidityStatus(ValidityStatusEnum.DEFINED_BY_VALIDITY_TIME_SPEC);
	  OverallPeriod overallPeriod = new OverallPeriod();
	  Date datumStart = new Date();
	  Calendar kalender = Calendar.getInstance();
	  kalender.setTime(datumStart);
	  kalender.add(Calendar.MILLISECOND, horizont);
	  Date datumEnd = kalender.getTime();
	  
	  XMLGregorianCalendar xmlGregorianCalendarStart = null;
	  XMLGregorianCalendar xmlGregorianCalendarEnd = null;
	  
	  xmlGregorianCalendarStart = WerkzeugSammlung.konvertiereDatum(datumStart);
	  xmlGregorianCalendarEnd = WerkzeugSammlung.konvertiereDatum(datumEnd);
	  
	  overallPeriod.setOverallStartTime(xmlGregorianCalendarStart);
	  overallPeriod.setOverallEndTime(xmlGregorianCalendarEnd);
	  validity.setValidityTimeSpecification(overallPeriod);
	  return validity;	    
  }
  
  /**
   * erzeugt einen Multilingualen String.
   * 
   * @param s
   *          Text fr den deutschen Text des Multiligualen Strings
   * @return MultilingualString
   */

protected static MultilingualString konvertiereString(final String s) {
	    MultilingualString ms = new MultilingualString();
	    Values vs = new Values();
	    MultilingualStringValue msv = new MultilingualStringValue();
	    msv.setLang("DE");
	    msv.setValue(s);
	    vs.getValues().add(msv);
	    ms.setValues(vs);
	    return ms;
	  }  

  @Override
  public void initialisiere(final Collection<ObjekteAtgAsp> objects) throws ConfigurationException {
    initialisiereBasisFelder();
    
    initialisiereStandardIntervall();    
  }
}
