package de.bsvrz.datex2.test.cases;
import java.io.ByteArrayOutputStream;
import java.io.StringReader;
import java.net.URL;

import javax.xml.XMLConstants;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.Marshaller;
import javax.xml.namespace.QName;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;

import org.junit.Assert;

import de.bsvrz.datex2.test.factories.CreateMockDataModel;
import de.bsvrz.datex2.test.factories.CreateMockMessquerschnitte;
import de.bsvrz.datex2.test.factories.CreateMockObjects;
import de.bsvrz.datex2.test.mockclasses.MockBrokenKonfigUndPersistenzModul;
import de.bsvrz.datex2.test.mockclasses.MockDataModel;
import de.bsvrz.datex2.test.mockclasses.MockDavConnection;
import de.bsvrz.datex2.test.mockclasses.MockErweiterterLogger;
import de.bsvrz.datex2.test.mockclasses.MockKonfigUndPersistenzModul;
import de.bsvrz.datex2.test.mockclasses.MockPublishMethod;
import de.bsvrz.dav.daf.main.MissingParameterException;
import de.bsvrz.dav.daf.main.config.ConfigurationChangeException;
import de.bsvrz.kex.datex2.plugins.definition.PluginExport;

public abstract class WhiteBox_PlugIns {
  
  private final String URI_PREFIX = "file:///d:/home/workspace/KExDatexII/org/";
  
  protected final String SCHEMA_URI_DYN;
  protected final String SCHEMA_URI_STAT;

  /** Datex II Namensraum */
  protected final String DATEX2_NAMESPACE = "http://datex2.eu/schema/2/2_0";
  /** Root-Tag des Datex II Formats */
  protected final String DATEX2_ROOTTAG = "d2LogicalModel";

  protected enum CanBeNull { CAN_BE_NULL, CANNOT_BE_NULL }

  public WhiteBox_PlugIns(final String schemaURIdyn, final String schemaURIstat) {
    this.SCHEMA_URI_DYN = URI_PREFIX + schemaURIdyn;
    this.SCHEMA_URI_STAT = URI_PREFIX + schemaURIstat;
  }

  abstract protected void populateConfigAndPersistence(MockKonfigUndPersistenzModul konfig, MockKonfigUndPersistenzModul persistenz);

  public void init(PluginExport mockPluginExport, MockDavConnection davConnection, CreateMockObjects objCreation,
      MockPublishMethod staticResult, MockPublishMethod dynamicResult) throws ConfigurationChangeException, MissingParameterException {
    CreateMockDataModel dataModelCreation = new CreateMockDataModel();
    MockDataModel dataModel = dataModelCreation.create();
    if (objCreation != null) {
      objCreation.setUsedDataModel(dataModel);
      objCreation.create();
    }

    MockKonfigUndPersistenzModul konfig = new MockKonfigUndPersistenzModul();
    MockKonfigUndPersistenzModul persistenz = new MockKonfigUndPersistenzModul();

    davConnection.setDataModel(dataModel);
    mockPluginExport.setKonfigurationsModul(konfig);
    mockPluginExport.setPersistenzModul(persistenz);    
    mockPluginExport.setDaVConnection(davConnection);
    mockPluginExport.setErweiterterLogger(new MockErweiterterLogger());

    konfig.putData("National-Identifier", "TestNI");
    konfig.putData("LocationGroupId", "TestGroupId");
    persistenz.putData("PredefinedLocationVersion", "correct");
    konfig.putData("StandardIntervall", "3.0");
    konfig.putData("Maximales-Alter", 3);
    persistenz.putData("HashCodeStaticXML", 123);

    populateConfigAndPersistence(konfig, persistenz);

    mockPluginExport.getDynamischePublizierungsMethoden().clear();
    mockPluginExport.getStatischePublizierungsMethoden().clear();
    if (dynamicResult != null) {
      mockPluginExport.getDynamischePublizierungsMethoden().add(dynamicResult);
    }
    if (staticResult != null) {
      mockPluginExport.getStatischePublizierungsMethoden().add(staticResult);    
    }
  }

  public void init(PluginExport mockPluginExport, MockDavConnection davConnection, CreateMockMessquerschnitte mqCreation) throws ConfigurationChangeException, MissingParameterException {
    init(mockPluginExport, davConnection, mqCreation, null, null);
  }

  public void init(PluginExport mockPluginExport) throws ConfigurationChangeException, MissingParameterException {
    CreateMockMessquerschnitte mqCreation = new CreateMockMessquerschnitte();
    MockDavConnection davConnection = new MockDavConnection(null, null);

    init(mockPluginExport, davConnection, mqCreation);
  }

  public boolean validate(String schemaFile, JAXBContext context, Object xml, CanBeNull canBeNull) throws Exception {
    URL schemaURL = new URL(schemaFile);
    SchemaFactory schemaFactory = SchemaFactory
        .newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
    Schema schema = schemaFactory.newSchema(schemaURL);

    if (xml == null) {
      return canBeNull.equals(CanBeNull.CAN_BE_NULL); 
    }
    Marshaller marshaller = context.createMarshaller();
    marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, new Boolean(true));

    marshaller.marshal(new JAXBElement<Object>(new QName(DATEX2_NAMESPACE,DATEX2_ROOTTAG), Object.class, xml),System.out);

    ByteArrayOutputStream ba = new ByteArrayOutputStream();
    marshaller.marshal(new JAXBElement<Object>(new QName(DATEX2_NAMESPACE,DATEX2_ROOTTAG), Object.class, xml), ba);
    Source xmls = new StreamSource(new StringReader(ba.toString()));
    Validator validator = schema.newValidator();
    validator.validate(xmls);

    return true;
  }

  /**
   * Fehlende obligatorische Parameter mssen zu einer Exception fhren.
   * 
   * @throws Exception
   */
  public void testFehlendeParameterStatisch(PluginExport pluginExport, String paramName) throws Exception { 
    init(pluginExport);

    MockKonfigUndPersistenzModul konfig = (MockKonfigUndPersistenzModul) pluginExport.getKonfigurationsModul();
    konfig.removeData(paramName);
    pluginExport.initialisiere();
  }

  /**
   * Fehlende optionale Parameter drfen nicht zu einer Nicht-Publikation fhren.
   * 
   * @throws Exception
   */
 
  public void testFehlendeOptionaleParameter(PluginExport pluginExport, String key) throws Exception {
    CreateMockMessquerschnitte mqCreation = new CreateMockMessquerschnitte();
    MockDavConnection davConnection = new MockDavConnection(null, null);
    MockPublishMethod dynamicResult = new MockPublishMethod();
    MockPublishMethod staticResult = new MockPublishMethod();

    init(pluginExport, davConnection, mqCreation, staticResult, dynamicResult);

    MockKonfigUndPersistenzModul konfig = (MockKonfigUndPersistenzModul) pluginExport.getKonfigurationsModul();
    konfig.removeData(key);

    pluginExport.initialisiere();

    pluginExport.getPluginAusfuehrungsAlgorithmus().starte();

    Assert.assertTrue(validate(SCHEMA_URI_STAT, pluginExport.getDatexKonvertierungStatisch().getJAXBContext(), staticResult.result , CanBeNull.CANNOT_BE_NULL));

    mqCreation.mockSendDataToAll(davConnection, 20, CreateMockObjects.SendOptions.NORMAL);

    pluginExport.getPluginAusfuehrungsAlgorithmus().aktualisiere();

    Assert.assertTrue(validate(SCHEMA_URI_DYN, pluginExport.getDatexKonvertierungDynamisch().getJAXBContext(), dynamicResult.result , CanBeNull.CANNOT_BE_NULL));

  }

  /**
   * Trotz fehlender Persistenz-Schicht sollten noch immer Daten abgeliefert werden. 
   * 
   * @throws Exception
   */
  
  public void testBrokenPersistence(PluginExport pluginExport, CreateMockObjects objCreation) throws Exception {
    MockDavConnection davConnection = new MockDavConnection(null, null);
    MockPublishMethod dynamicResult = new MockPublishMethod();
    MockPublishMethod staticResult = new MockPublishMethod();

    init(pluginExport, davConnection, objCreation, staticResult, dynamicResult);

    pluginExport.setPersistenzModul(new MockBrokenKonfigUndPersistenzModul());

    pluginExport.initialisiere();

    pluginExport.getPluginAusfuehrungsAlgorithmus().starte();

    Assert.assertTrue(validate(SCHEMA_URI_STAT, pluginExport.getDatexKonvertierungStatisch().getJAXBContext(), staticResult.result , CanBeNull.CAN_BE_NULL));

    objCreation.mockSendValidDataToAll(davConnection);

    pluginExport.getPluginAusfuehrungsAlgorithmus().aktualisiere();

    Assert.assertTrue(validate(SCHEMA_URI_DYN, pluginExport.getDatexKonvertierungDynamisch().getJAXBContext(), dynamicResult.result , CanBeNull.CANNOT_BE_NULL));
  }

  /**
   * Wenn keine Umfelddatensensoren und -messstellen definiert sind, kann zwar kein XML erzeugt werden (= leeres Dokument),
   * die Exceptions mssen aber in den aufgerufenen Methoden abgefangen werden.
   * 
   * @throws Exception
   */
  
  public void testKeineObjekte(PluginExport pluginExport) throws Exception {
    MockDavConnection davConnection = new MockDavConnection(null, null);
    MockPublishMethod dynamicResult = new MockPublishMethod();
    MockPublishMethod staticResult = new MockPublishMethod();

    init(pluginExport, davConnection, null, staticResult, dynamicResult);

    pluginExport.initialisiere();

    pluginExport.getPluginAusfuehrungsAlgorithmus().starte();

    Assert.assertTrue(validate(SCHEMA_URI_STAT, pluginExport.getDatexKonvertierungStatisch().getJAXBContext(), staticResult.result , CanBeNull.CAN_BE_NULL));

    pluginExport.getPluginAusfuehrungsAlgorithmus().aktualisiere();

    Assert.assertTrue(validate(SCHEMA_URI_DYN, pluginExport.getDatexKonvertierungDynamisch().getJAXBContext(), dynamicResult.result , CanBeNull.CAN_BE_NULL));    
  }

}