package de.bsvrz.datex2.test.cases.old;
import java.io.ByteArrayOutputStream;
import java.io.StringReader;
import java.lang.reflect.Field;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;

import javax.naming.ConfigurationException;
import javax.xml.XMLConstants;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.Marshaller;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.namespace.QName;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import de.bsvrz.datex2.test.mockclasses.MockAttributeGroup;
import de.bsvrz.datex2.test.mockclasses.MockData;
import de.bsvrz.datex2.test.mockclasses.MockDataModel;
import de.bsvrz.datex2.test.mockclasses.MockDataNumberValue;
import de.bsvrz.datex2.test.mockclasses.MockDataTextValue;
import de.bsvrz.datex2.test.mockclasses.MockKonfigUndPersistenzModul;
import de.bsvrz.datex2.test.mockclasses.MockObjekteAtgAsp;
import de.bsvrz.datex2.test.mockclasses.MockPluginExport;
import de.bsvrz.datex2.test.mockclasses.MockSystemObject;
import de.bsvrz.kex.datex2.allgemein.ObjekteAtgAsp;
import de.bsvrz.kex.datex2.plugins.konvertierung.impl.MQPredefinedLocation;
import de.bsvrz.kex.datex2.plugins.konvertierung.impl.WerkzeugSammlung;

public class WBKonvertierung001s {

  private static final String TYP_MESSQUERSCHNITT = "typ.messQuerschnitt";
  private static final String ATG_VERKEHRSDATEN = "atg.verkehrsDatenKurzZeitMq";
  private static final String SCHEMA_URI = "file:///d:/home/workspace/KExDatexII/org/DATEXII_DaV-MDM-001_stat.xsd";
  
  /** Datex II Namensraum */
  private final String DATEX2_NAMESPACE = "http://datex2.eu/schema/2/2_0";

  /** Root-Tag des Datex II Formats */
  private final String DATEX2_ROOTTAG = "d2LogicalModel";

  MockObjekteAtgAsp ordinaryObjSet = new MockObjekteAtgAsp();
  MockObjekteAtgAsp faultObjSet1 = new MockObjekteAtgAsp();
  MockObjekteAtgAsp faultObjSet2 = new MockObjekteAtgAsp();
  MockObjekteAtgAsp faultObjSet3 = new MockObjekteAtgAsp();
  MockObjekteAtgAsp faultObjSet4 = new MockObjekteAtgAsp();
  MockObjekteAtgAsp emptyObjSet = new MockObjekteAtgAsp();
  private Schema schema;
  
  enum CanBeNull { CAN_BE_NULL, CANNOT_BE_NULL };

  @Before
  public void createDataSets() throws Exception {
    
    MockDataModel mockDataModel;

    mockDataModel = new MockDataModel();
    
    MockAttributeGroup atgPunkt = new MockAttributeGroup("atg.punktKoordinaten");
    
    mockDataModel.getAttributeGroupMap().put("atg.punktKoordinaten", atgPunkt);

    ordinaryObjSet = new MockObjekteAtgAsp();
    ordinaryObjSet.createMockTyp(TYP_MESSQUERSCHNITT, "Messquerschnitt");
    ordinaryObjSet.createAttributeGroup(ATG_VERKEHRSDATEN, "Verkehrsdaten");
    MockSystemObject so = new MockSystemObject("pid.mq01", "");
    so.setDataModel(mockDataModel);
    MockData dp = new MockData();
    MockDataNumberValue nv = new MockDataNumberValue();
    nv.set(34);
    dp.setNumberValue("x", nv);
    dp.setNumberValue("y", nv);
    so.getConfigurationDataMap().put(atgPunkt.getPid(), dp);
 
    MockSystemObject so2 = new MockSystemObject("pid.mq02", "");
    so2.setDataModel(mockDataModel);
    so2.getConfigurationDataMap().put(atgPunkt.getPid(), dp);
    faultObjSet1.getSystemObjekte().add(so);

    ordinaryObjSet.getSystemObjekte().add(so);
    ordinaryObjSet.getSystemObjekte().add(so2);
    
    faultObjSet1 = new MockObjekteAtgAsp();
    faultObjSet1.createMockTyp("typ.nichtExistent", "Messquerschnitt");
    faultObjSet1.createAttributeGroup(ATG_VERKEHRSDATEN, "Verkehrsdaten");

    faultObjSet2 = new MockObjekteAtgAsp();
    faultObjSet2.createMockTyp(TYP_MESSQUERSCHNITT, "Messquerschnitt");
    faultObjSet2.createAttributeGroup("atg.nichtExistent", "Verkehrsdaten");
    
    faultObjSet3 = new MockObjekteAtgAsp();
    faultObjSet3.createMockTyp(TYP_MESSQUERSCHNITT, "Messquerschnitt");
    faultObjSet3.createAttributeGroup("atg.nichtExistent", "Verkehrsdaten");
    MockSystemObject so3 = new MockSystemObject("pid.mq03", "");
    so3.setDataModel(mockDataModel);
    faultObjSet3.getSystemObjekte().add(so3);
    System.err.println("so3: " + so3.getConfigurationDataMap());

    faultObjSet4 = new MockObjekteAtgAsp();
    faultObjSet4.createMockTyp(TYP_MESSQUERSCHNITT, "Messquerschnitt");
    faultObjSet4.createAttributeGroup(ATG_VERKEHRSDATEN, "Verkehrsdaten");
    MockSystemObject so4 = new MockSystemObject("pid.mq04", "");
    so4.setDataModel(mockDataModel);
    MockData dp4 = new MockData();
    MockDataTextValue nv4 = new MockDataTextValue();
    nv4.setText("HAHA");
    dp4.setTextValue("x", nv4);
    dp4.setTextValue("y", nv4);
    so4.getConfigurationDataMap().put(atgPunkt.getPid(), dp4);

    faultObjSet4.getSystemObjekte().add(so4);
    faultObjSet4.getSystemObjekte().add(so2);
        
    emptyObjSet = new MockObjekteAtgAsp();

    URL schemaFile = new URL(SCHEMA_URI);
    SchemaFactory schemaFactory = SchemaFactory
        .newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
    schema = schemaFactory.newSchema(schemaFile);
  }
    
  public Object apply(MockPluginExport plugin,ObjekteAtgAsp objekteAtgAsp) {
    Collection<ObjekteAtgAsp> objList = new ArrayList<>();
    objList.add(objekteAtgAsp);
    for (ObjekteAtgAsp a : objList) {
      System.err.println("r " + a.getSystemObjekte().size());
    }

    try {
      plugin.getDatexKonvertierungStatisch().initialisiere(objList);
    } catch (ConfigurationException e) {
      e.printStackTrace();
      return null;
    }
    
    for (ObjekteAtgAsp a : objList) {
      System.err.println(a.getSystemObjekte().size());
    }
    
    return plugin.getDatexKonvertierungStatisch().getDatexII(objList);
  }
  
  public MockPluginExport setupPluginFull() {
    MockPluginExport mockPluginExport = new MockPluginExport();
    MockKonfigUndPersistenzModul konfig = new MockKonfigUndPersistenzModul();
    MockKonfigUndPersistenzModul persistenz = new MockKonfigUndPersistenzModul();
    mockPluginExport.setKonfigurationsModul(konfig);
    mockPluginExport.setPersistenzModul(persistenz);    
    
    mockPluginExport.setDatexKonvertierungStatisch(new MQPredefinedLocation(mockPluginExport));
        
    konfig.putData("National-Identifier", "TestNI");
    konfig.putData("LocationGroupId", "TestGroupId");
    konfig.putData("StandardIntervall", "3.0");
    
    return mockPluginExport;
  }
  
  @Test
  public void test001() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    Object xml = apply(plugin, ordinaryObjSet);
    System.err.println(xml);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CANNOT_BE_NULL));
  }

  @Test
  public void test002() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    Object xml = apply(plugin, faultObjSet1);
    System.err.println(xml);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CAN_BE_NULL));
  }

  @Test
  public void test003() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    Object xml = apply(plugin, faultObjSet2);
    System.err.println(xml);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CAN_BE_NULL));
  }

  @Test
  public void test004() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    Object xml = apply(plugin, faultObjSet3);
    System.err.println(xml);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CAN_BE_NULL));
  }

  @Test
  public void test005() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    Object xml = apply(plugin, faultObjSet4);
    System.err.println(xml);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CAN_BE_NULL));
  }
  
  @Test
  public void test011() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    ((MockKonfigUndPersistenzModul) plugin.getPersistenzModul()).putData("PredefinedLocationVersion", "MOCKVERSION");
    Object xml = apply(plugin, faultObjSet1);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CAN_BE_NULL));
    Assert.assertEquals("MOCKVERSION", ((MQPredefinedLocation) plugin.getDatexKonvertierungStatisch()).getPredefinedLocationVersion()); 
  }
  
  @Test
  public void test012() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    ((MockKonfigUndPersistenzModul) plugin.getKonfigurationsModul()).removeData("National-Identifier");
    Object xml = apply(plugin, ordinaryObjSet);
    Assert.assertNull(xml);
  }
  
  @Test
  public void test013() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    ((MockKonfigUndPersistenzModul) plugin.getKonfigurationsModul()).removeData("LocationGroupId");
    Object xml = apply(plugin, ordinaryObjSet);
    Assert.assertNull(xml);
  }

  @Test
  public void test017() throws Exception {
    for (Field f : WerkzeugSammlung.class.getDeclaredFields()) {
      if (f.getType().equals(DatatypeFactory.class)) {
        f.setAccessible(true);
        f.set(null,null);
        System.err.println("Reflection: Found and patched field.");
      }
    }
 
    System.getProperties().put("javax.xml.datatype.DatatypeFactory", "Object");
    MockPluginExport plugin = setupPluginFull();
    Object xml = apply(plugin, ordinaryObjSet);
    System.getProperties().remove("javax.xml.datatype.DatatypeFactory");
    Assert.assertNull(xml);
  }

  public boolean validate(MockPluginExport plugin, Object xml, CanBeNull canBeNull) throws Exception {
    if (xml == null) {
      return canBeNull.equals(CanBeNull.CAN_BE_NULL); 
    }
    Marshaller marshaller = plugin.getDatexKonvertierungStatisch().getJAXBContext().createMarshaller();
    marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, new Boolean(true));

    marshaller.marshal(new JAXBElement<Object>(new QName(DATEX2_NAMESPACE,DATEX2_ROOTTAG), Object.class, xml),System.out);

    ByteArrayOutputStream ba = new ByteArrayOutputStream();
    marshaller.marshal(new JAXBElement<Object>(new QName(DATEX2_NAMESPACE,DATEX2_ROOTTAG), Object.class, xml), ba);
    Source xmls = new StreamSource(new StringReader(ba.toString()));
    Validator validator = schema.newValidator();
    validator.validate(xmls);

    return true;
  }
}
