package de.bsvrz.datex2.test.cases.old;
import java.io.ByteArrayOutputStream;
import java.io.StringReader;
import java.lang.reflect.Field;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.naming.ConfigurationException;
import javax.xml.XMLConstants;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.Marshaller;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.namespace.QName;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import de.bsvrz.datex2.test.mockclasses.MockArray;
import de.bsvrz.datex2.test.mockclasses.MockData;
import de.bsvrz.datex2.test.mockclasses.MockDataModel;
import de.bsvrz.datex2.test.mockclasses.MockDataNumberValue;
import de.bsvrz.datex2.test.mockclasses.MockDataTextValue;
import de.bsvrz.datex2.test.mockclasses.MockDataTimeValue;
import de.bsvrz.datex2.test.mockclasses.MockKonfigUndPersistenzModul;
import de.bsvrz.datex2.test.mockclasses.MockObjekteAtgAsp;
import de.bsvrz.datex2.test.mockclasses.MockPluginExport;
import de.bsvrz.datex2.test.mockclasses.MockSystemObject;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.kex.datex2.allgemein.ObjekteAtgAsp;
import de.bsvrz.kex.datex2.plugins.konvertierung.impl.RSPredefinedLocation;
import de.bsvrz.kex.datex2.plugins.konvertierung.impl.RSReisezeit;
import de.bsvrz.kex.datex2.plugins.konvertierung.impl.UDMeasurementSite;
import de.bsvrz.kex.datex2.plugins.konvertierung.impl.WerkzeugSammlung;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;

public class WBKonvertierung006d {

  private static final String TYP_MESSQUERSCHNITT = "typ.ueresStraenSegment";
  private static final String ATG_VERKEHRSDATEN = "atg.strfallVerkehrsZustand";
  
  private static final String SCHEMA_URI = "file:///d:/home/workspace/KExDatexII/org/DATEXII_DaV-MDM-006_dyn.xsd";
  
  /** Datex II Namensraum */
  private final String DATEX2_NAMESPACE = "http://datex2.eu/schema/2/2_0";

  /** Root-Tag des Datex II Formats */
  private final String DATEX2_ROOTTAG = "d2LogicalModel";

  Collection<ResultData> ordinaryDataset = new ArrayList<ResultData>();
  Collection<ResultData> boundaryDataset = new ArrayList<ResultData>();
  Collection<ResultData> faultDataset1 = new ArrayList<ResultData>();
  Collection<ResultData> faultDataset2 = new ArrayList<ResultData>();
  Collection<ResultData> faultDataset3 = new ArrayList<ResultData>();
  Collection<ResultData> faultDataset4 = new ArrayList<ResultData>();
  Collection<ResultData> faultDataset5 = new ArrayList<ResultData>();
  Collection<ResultData> faultDataset6 = new ArrayList<ResultData>();
  Collection<ResultData> faultDataset7 = new ArrayList<ResultData>();
  Collection<ResultData> emptyDataset = new ArrayList<ResultData>();

  MockObjekteAtgAsp ordinaryObjSet = new MockObjekteAtgAsp();
  MockObjekteAtgAsp faultObjSet1 = new MockObjekteAtgAsp();
  MockObjekteAtgAsp faultObjSet2 = new MockObjekteAtgAsp();
  MockObjekteAtgAsp emptyObjSet = new MockObjekteAtgAsp();
  
  MockDataModel mockDataModel;
  
  private Schema schema;
  
  enum CanBeNull { CAN_BE_NULL, CANNOT_BE_NULL };
  
  @Before
  public void createDataSets() throws Exception {

    ordinaryDataset.add(createOrdinaryDataset1());
    ordinaryDataset.add(createOrdinaryDataset1());

    faultDataset1.add(createFaultDataset1());
    faultDataset2.add(createFaultDataset2());
    faultDataset3.add(createFaultDataset3());
    faultDataset4.add(createFaultDataset4());
    faultDataset5.add(createFaultDataset5());
    faultDataset6.add(createFaultDataset6());
    faultDataset7.add(null);

    ordinaryObjSet = new MockObjekteAtgAsp();
    ordinaryObjSet.createMockTyp(TYP_MESSQUERSCHNITT, "Messquerschnitt");
    ordinaryObjSet.createAttributeGroup(ATG_VERKEHRSDATEN, "Verkehrsdaten");
    ordinaryObjSet.getSystemObjekte().add(new MockSystemObject("pid.test.mq.ordinary01", ""));


    faultObjSet1 = new MockObjekteAtgAsp();
    faultObjSet1.createMockTyp("typ.nichtExistent", "Messquerschnitt");
    faultObjSet1.createAttributeGroup(ATG_VERKEHRSDATEN, "Verkehrsdaten");

    faultObjSet2 = new MockObjekteAtgAsp();
    faultObjSet2.createMockTyp(TYP_MESSQUERSCHNITT, "Messquerschnitt");
    faultObjSet2.createAttributeGroup("atg.nichtExistent", "Verkehrsdaten");
    
    emptyObjSet = new MockObjekteAtgAsp();

    URL schemaFile = new URL(SCHEMA_URI);
    SchemaFactory schemaFactory = SchemaFactory
        .newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
    schema = schemaFactory.newSchema(schemaFile);
  }

  public ResultData createOrdinaryDataset1() {
    MockArray a = new MockArray();
    MockData rsx = new MockData();
    rsx.addItem("ReisezeitPrognose", a);
    
    MockData rd = new MockData();
    rd.addItem("Reisezeit", new MockDataTimeValue(2122));
    rd.addItem("ReisezeitAlg2", new MockDataTimeValue(2122));
    rd.addItem("Zeitpunkt", new MockDataTimeValue(1212323));
    a.items.add(rd);
    SystemObject obj = new MockSystemObject("pid.mock.ordinary1", "");
    return new ResultData(obj, null, false, 0L, 0L, (byte) 0, rsx);
  }
    
  public ResultData createBoundaryDataset() {
    MockData data = new MockData();
    MockDataTextValue situation = new MockDataTextValue();
    MockDataTimeValue horizont = new MockDataTimeValue();
    MockDataTimeValue T = new MockDataTimeValue();
    MockDataTimeValue refZeit = new MockDataTimeValue();
    MockDataTimeValue travelTime = new MockDataTimeValue();
    MockData guete = new MockData();
    MockDataNumberValue trafficv = new MockDataNumberValue();
    trafficv.set(0);
    MockDataNumberValue index = new MockDataNumberValue();
    index.set(0);
    guete.setNumberValue("Index", index);
    situation.setText("Stau");
    horizont.setMillis(0);
    T.setMillis(0);
    travelTime.setMillis(999);
    refZeit.setMillis(new Date().getTime());
    data.setTextValue("Situation", situation);
    data.setTimeValue("PrognoseHorizont", horizont);
    data.setTimeValue("T", T);
    data.setTimeValue("ReisezeitKfz", travelTime);
    data.setTimeValue("ReisezeitPkw", travelTime);
    data.setTimeValue("ReisezeitLkw", travelTime);
    data.setNumberValue("DichteLkw", trafficv);
    data.setNumberValue("DichtePkw", trafficv);
    data.setNumberValue("DichteKfz", trafficv);
    data.setNumberValue("FlussLkw", trafficv);
    data.setNumberValue("FlussPkw", trafficv);
    data.setNumberValue("FlussKfz", trafficv);
    data.setNumberValue("vLkw", trafficv);
    data.setNumberValue("vPkw", trafficv);
    data.setNumberValue("vKfz", trafficv);
    data.setTimeValue("ReferenzZeit", refZeit);
      data.addItem("Gte", guete);
    SystemObject obj = new MockSystemObject("pid.mock.ordinary1", "");
    return new ResultData(obj, null, false, 0L, 0L, (byte) 0, data);
  }

  // Unpassende Attributgruppe: Keine Werte abrufbar
  public ResultData createFaultDataset1() {
    MockData data = new MockData();
    MockDataTextValue situation = new MockDataTextValue();
    MockDataTimeValue horizont = new MockDataTimeValue();
    MockDataTimeValue T = new MockDataTimeValue();
    MockDataTimeValue refZeit = new MockDataTimeValue();
    MockData guete = new MockData();
    MockDataNumberValue trafficv = new MockDataNumberValue();
    trafficv.set(0);
    MockDataNumberValue index = new MockDataNumberValue();
    index.set(-1);
    guete.setNumberValue("Index", index);
    situation.setText("Stau");
    horizont.setMillis(0);
    T.setMillis(0);
    refZeit.setMillis(new Date().getTime());
    data.setTextValue("Situation", situation);
    data.setTimeValue("PrognoseHorizont", horizont);
    data.setTimeValue("T", T);
    data.setTimeValue("ReferenzZeit", refZeit);
      data.addItem("Gte", guete);
    SystemObject obj = new MockSystemObject("pid.mock.ordinary1", "");
    return new ResultData(obj, null, false, 0L, 0L, (byte) 0, data);
  }

  // Fehlerhafte Werte (negativ)
  public ResultData createFaultDataset2() {
    MockData data = new MockData();
    MockData values = new MockData();
    MockDataNumberValue numericValues = new MockDataNumberValue();
    numericValues.set(-1);
    data.addItem("QPkw", values);
    data.addItem("QLkw", values);
    data.addItem("QKfz", values);
    data.addItem("VPkw", values);
    data.addItem("VLkw", values);
    data.addItem("VKfz", values);
    data.addItem("ALkw", values);
    values.setNumberValue("Wert",numericValues);
    data.addItem("SomethingDifferent", values);
    SystemObject obj = new MockSystemObject("pid.mock.fault1", "");
    return new ResultData(obj, null, false, 0L, 0L, (byte) 0, data);
  }

  // Fehlerhafte Werte (zu gro)
  public ResultData createFaultDataset3() {
    MockData data = new MockData();
    MockData values = new MockData();
    MockDataNumberValue numericValues = new MockDataNumberValue();
    numericValues.set(10000);
    data.addItem("QPkw", values);
    data.addItem("QLkw", values);
    data.addItem("QKfz", values);
    data.addItem("VPkw", values);
    data.addItem("VLkw", values);
    data.addItem("VKfz", values);
    data.addItem("ALkw", values);
    values.setNumberValue("Wert",numericValues);
    data.addItem("SomethingDifferent", values);
    SystemObject obj = new MockSystemObject("pid.mock.fault1", "");
    return new ResultData(obj, null, false, 0L, 0L, (byte) 0, data);
  }

  // Fehlerhafte Werte (Fehlertexte)
  public ResultData createFaultDataset4() {
    MockData data = new MockData();
    MockData values = new MockData();
    MockDataNumberValue numericValues = new MockDataNumberValue();
    numericValues.set(10000);
    data.addItem("QPkw", values);
    data.addItem("QLkw", values);
    data.addItem("QKfz", values);
    data.addItem("VPkw", values);
    data.addItem("VLkw", values);
    data.addItem("VKfz", values);
    data.addItem("ALkw", values);
    values.setNumberValue("Wert",numericValues);
    data.addItem("SomethingDifferent", values);
    SystemObject obj = new MockSystemObject("pid.mock.fault1", "");
    return new ResultData(obj, null, false, 0L, 0L, (byte) 0, data);
  }

  // Fehlerhafte Werte (hasData == false)
  public ResultData createFaultDataset5() {
    MockData data = new MockData();
    MockData values = new MockData();
    MockDataNumberValue numericValues = new MockDataNumberValue();
    numericValues.set(45);
    data.addItem("QPkw", values);
    data.addItem("QLkw", values);
    data.addItem("QKfz", values);
    data.addItem("VPkw", values);
    data.addItem("VLkw", values);
    data.addItem("VKfz", values);
    data.addItem("ALkw", values);
    values.setNumberValue("Wert",numericValues);
    data.addItem("SomethingDifferent", values);
    SystemObject obj = new MockSystemObject("pid.mock.fault1", "");
    return new ResultData(obj, null, false, 0L, 0L, (byte) 1, data);
  }

  // Fehlerhaftes ResultData (object == null)
  public ResultData createFaultDataset6() {
    MockData data = new MockData();
    MockData values = new MockData();
    MockDataNumberValue numericValues = new MockDataNumberValue();
    numericValues.set(45);
    values.setNumberValue("Wert",numericValues);
    data.addItem("QPkw", values);
    data.addItem("QLkw", values);
    data.addItem("QKfz", values);
    data.addItem("VPkw", values);
    data.addItem("VLkw", values);
    data.addItem("VKfz", values);
    data.addItem("ALkw", values);
    return new ResultData(null, null, false, 0L, 0L, (byte) 0, data);
  }

  public Object apply(MockPluginExport plugin,ObjekteAtgAsp objekteAtgAsp, Collection<ResultData> resultData) {
    Collection<ObjekteAtgAsp> objList = new ArrayList<>();
    objList.add(objekteAtgAsp);

    try {
      plugin.getDatexKonvertierungDynamisch().initialisiere(objList);
    } catch (ConfigurationException ex) {
      return null;
    }

    String[] s = new String[0];
    
    ArgumentList al = new ArgumentList(s);
    Debug.init("WhiteBox", al);

    Debug.setLoggerLevel("", Debug.FINE);
        
    Map<ObjekteAtgAsp, Collection<ResultData> > data = new HashMap<>();

    data.put(objekteAtgAsp, resultData);
        
    return plugin.getDatexKonvertierungDynamisch().getDatexII(data);
  }
  
  public MockPluginExport setupPluginFull() {
    MockPluginExport mockPluginExport = new MockPluginExport();
    MockKonfigUndPersistenzModul konfig = new MockKonfigUndPersistenzModul();
    MockKonfigUndPersistenzModul persistenz = new MockKonfigUndPersistenzModul();
    mockPluginExport.setKonfigurationsModul(konfig);
    mockPluginExport.setPersistenzModul(persistenz);    
    
    mockPluginExport.setDatexKonvertierungDynamisch(new RSReisezeit(mockPluginExport));
    mockPluginExport.setDatexKonvertierungStatisch(new RSPredefinedLocation(mockPluginExport));
           
    konfig.putData("National-Identifier", "TestNI");
    konfig.putData("LocationGroupId", "TestGroupId");
    konfig.putData("StandardIntervall", "3.0");
    
    return mockPluginExport;
  }
  
  @Test
  public void test001() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    Object xml = apply(plugin, ordinaryObjSet, ordinaryDataset);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CANNOT_BE_NULL));
  }


  @Test
  public void test004() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    Object xml = apply(plugin, ordinaryObjSet, faultDataset2);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CAN_BE_NULL));
  }

  @Test
  public void test005() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    Object xml = apply(plugin, ordinaryObjSet, faultDataset3);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CAN_BE_NULL));
  }

  @Test
  public void test006() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    Object xml = apply(plugin, ordinaryObjSet, faultDataset4);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CAN_BE_NULL));
  }

  @Test
  public void test007() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    Object xml = apply(plugin, ordinaryObjSet, faultDataset5);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CAN_BE_NULL));
  }

  @Test
  public void test008() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    Object xml = apply(plugin, faultObjSet1, ordinaryDataset);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CAN_BE_NULL));
  }

  @Test
  public void test009() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    Object xml = apply(plugin, faultObjSet2, ordinaryDataset);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CAN_BE_NULL));
  }

  @Test
  public void test010() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    Object xml = apply(plugin, ordinaryObjSet, emptyDataset);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CAN_BE_NULL));
  }
  

  @Test
  public void test011() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    Object xml = apply(plugin, ordinaryObjSet, faultDataset6);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CAN_BE_NULL));
  }
  
  @Test
  public void test012() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    ((MockKonfigUndPersistenzModul) plugin.getKonfigurationsModul()).removeData("National-Identifier");
    Object xml = apply(plugin, ordinaryObjSet, ordinaryDataset);
    Assert.assertNull(xml);
  }

  @Test
  public void test013() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    ((MockKonfigUndPersistenzModul) plugin.getKonfigurationsModul()).removeData("LocationGroupId");
    Object xml = apply(plugin, ordinaryObjSet, ordinaryDataset);
    Assert.assertNull(xml);
  }
  
  @Test
  public void test014() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    ((MockKonfigUndPersistenzModul) plugin.getKonfigurationsModul()).removeData("StandardIntervall");
    Object xml = apply(plugin, ordinaryObjSet, ordinaryDataset);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CANNOT_BE_NULL));
  }

  @Test
  public void test016() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    ((MockKonfigUndPersistenzModul) plugin.getKonfigurationsModul()).removeData("StandardIntervall");
    ((MockKonfigUndPersistenzModul) plugin.getKonfigurationsModul()).putData("StandardIntervall", "QWERT");
    Object xml = apply(plugin, ordinaryObjSet, ordinaryDataset);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CANNOT_BE_NULL));
  }

  @Test
  public void test017() throws Exception {
    for (Field f : WerkzeugSammlung.class.getDeclaredFields()) {
      if (f.getType().equals(DatatypeFactory.class)) {
        f.setAccessible(true);
        f.set(null,null);
        System.err.println("Reflection: Found and patched field.");
      }
    }
  
    System.getProperties().put("javax.xml.datatype.DatatypeFactory", "Object");
    MockPluginExport plugin = setupPluginFull();
    Object xml = apply(plugin, ordinaryObjSet, ordinaryDataset);
    Assert.assertNull(xml);
    System.getProperties().remove("javax.xml.datatype.DatatypeFactory");
  }

  @Test
  public void test018() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    Object xml = apply(plugin, ordinaryObjSet, faultDataset7);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CAN_BE_NULL));
  }

  @Test
  public void test019() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    plugin.setDatexKonvertierungStatisch(null);
    Object xml = apply(plugin, ordinaryObjSet, ordinaryDataset);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CANNOT_BE_NULL));
  }

  @Test
  public void test020() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    plugin.setDatexKonvertierungStatisch(null);
    Object xml = apply(plugin, ordinaryObjSet, ordinaryDataset);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CANNOT_BE_NULL));
  }

  @Test
  public void test021() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    Field f = plugin.getDatexKonvertierungStatisch().getClass().getDeclaredField("version");
    f.setAccessible(true);
    f.set(plugin.getDatexKonvertierungStatisch(), null);
    Object xml = apply(plugin, ordinaryObjSet, ordinaryDataset);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CANNOT_BE_NULL));
  }

  @Test
  public void test022() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    Field f = plugin.getDatexKonvertierungStatisch().getClass().getDeclaredField("version");
    f.setAccessible(true);
    f.set(plugin.getDatexKonvertierungStatisch(), "");
    Object xml = apply(plugin, ordinaryObjSet, ordinaryDataset);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CANNOT_BE_NULL));
  }

  @Test
  public void test023() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    plugin.setDatexKonvertierungStatisch(new UDMeasurementSite(plugin));
    Object xml = apply(plugin, ordinaryObjSet, ordinaryDataset);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CANNOT_BE_NULL));
  }


  public boolean validate(MockPluginExport plugin, Object xml, CanBeNull canBeNull) throws Exception {
    if (xml == null) {
      return canBeNull.equals(CanBeNull.CAN_BE_NULL); 
    }
    Marshaller marshaller = plugin.getDatexKonvertierungDynamisch().getJAXBContext().createMarshaller();
    marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, new Boolean(true));

    marshaller.marshal(new JAXBElement<Object>(new QName(DATEX2_NAMESPACE,DATEX2_ROOTTAG), Object.class, xml),System.out);

    ByteArrayOutputStream ba = new ByteArrayOutputStream();
    marshaller.marshal(new JAXBElement<Object>(new QName(DATEX2_NAMESPACE,DATEX2_ROOTTAG), Object.class, xml), ba);
    Source xmls = new StreamSource(new StringReader(ba.toString()));
    Validator validator = schema.newValidator();
    validator.validate(xmls);

    return true;
  }
}
