package de.bsvrz.datex2.test.factories;
import java.util.Date;
import java.util.List;

import de.bsvrz.datex2.test.mockclasses.MockData;
import de.bsvrz.datex2.test.mockclasses.MockDataModel;
import de.bsvrz.datex2.test.mockclasses.MockDataNumberValue;
import de.bsvrz.datex2.test.mockclasses.MockDataTextValue;
import de.bsvrz.datex2.test.mockclasses.MockDavConnection;
import de.bsvrz.datex2.test.mockclasses.MockSystemObject;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationChangeException;
import de.bsvrz.dav.daf.main.config.SystemObject;

public abstract class CreateMockObjects {

  protected MockDataModel usedDataModel;

  public enum SendOptions {
    NORMAL, NULL_OBJ, HAS_NO_DATA
  };
  
  public CreateMockObjects() {
    super();
  }

  public CreateMockObjects(final MockDataModel dataModel) {
    this.usedDataModel = dataModel;
  }

  public MockDataModel getUsedDataModel() {
    return usedDataModel;
  }

  public void setUsedDataModel(MockDataModel usedDataModel) {
    this.usedDataModel = usedDataModel;
  }

  public MockSystemObject createLocatedObject(String pid, String typ, int locationX, int locationY) {
    MockSystemObject mq01 = new MockSystemObject(pid, typ);
    mq01.setDataModel(usedDataModel);
    usedDataModel.addObject(usedDataModel.getType(typ), mq01);
    
    MockData locationData = new MockData();
    MockDataNumberValue locationDataValueX = new MockDataNumberValue();
    MockDataNumberValue locationDataValueY = new MockDataNumberValue();
    locationDataValueX.set(locationX);
    locationDataValueY.set(locationY);
    locationData.setNumberValue("x", locationDataValueX);
    locationData.setNumberValue("y", locationDataValueY);
    
    AttributeGroup atgPunkt = usedDataModel.getAttributeGroupMap().get("atg.punktKoordinaten");
  
    mq01.getConfigurationDataMap().put(atgPunkt.getPid(), locationData);
  
    return mq01;
  }

  public void mockSendNumericData(MockDavConnection davConnection, String typ, String item, String key, Number number) {
    MockData mockData = createNumberData(item, key, number);
    mockSendData(davConnection, typ, mockData);
  }

  public void mockSendNumericData(MockDavConnection davConnection, String typ, List<String> item, String key, Number number, SendOptions sendOptions) {
    MockData mockData = createNumberData(item, key, number);
    mockSendData(davConnection, typ, mockData, sendOptions);
  }

  public void mockSendData(MockDavConnection davConnection, String typ, MockData data) {
    mockSendData(davConnection, typ, data, SendOptions.NORMAL);
  }
  
  public void mockSendData(MockDavConnection davConnection, String typ, MockData data, SendOptions sendOptions) {
    ResultData[] rdList = new ResultData[1];
    
    for (SystemObject obj : usedDataModel.getObjects(typ, 0, 0)) {
      if (SendOptions.NULL_OBJ.equals(sendOptions)) {
        rdList[0] = new ResultData(null, null, false, 0L, 0L, (byte) 0, data);
      } else if (SendOptions.HAS_NO_DATA.equals(sendOptions)) {
        rdList[0] = new ResultData(obj, null, false, 0L, 0L, (byte) 1, data);       
      } else {
        rdList[0] = new ResultData(obj, null, false, 0L, 0L, (byte) 0, data);
      }
      rdList[0].setDataTime(new Date().getTime());
      
      for (ClientReceiverInterface i : davConnection.getReceiver(obj)) {
        i.update(rdList);
      }
    }
  }

  public MockData createNumberData(final String item, final String key, final Number value) {
    MockData dataLevel1 = new MockData();
    MockData dataLevel2 = new MockData();
    MockDataNumberValue dataLevel3 = new MockDataNumberValue(value);
    dataLevel2.setNumberValue(key, dataLevel3);
    dataLevel1.addItem(item, dataLevel2);
    
    return dataLevel1;
  }
  
  public MockData createNumberData(final List<String> items, final String key, final Number value) {
    MockData dataLevel1 = new MockData();
    MockData dataLevel2 = new MockData();
    MockDataNumberValue dataLevel3 = new MockDataNumberValue(value);
    dataLevel2.setNumberValue(key, dataLevel3);
    for (String item : items) {
      dataLevel1.addItem(item, dataLevel2);
    }
    return dataLevel1;
  }

  public MockData createTextData(final String item, final String key, final String value) {
    MockData dataLevel1 = new MockData();
    MockData dataLevel2 = new MockData();
    MockDataTextValue dataLevel3 = new MockDataTextValue();
    dataLevel3.setText(value);
    dataLevel2.setTextValue(key, dataLevel3);
    dataLevel1.addItem(item, dataLevel2);
    
    return dataLevel1;
  }

  public abstract void mockSendValidDataToAll(MockDavConnection davConnection);

  public abstract void create() throws ConfigurationChangeException;
  
}