package de.bsvrz.datex2.test.factories;
import de.bsvrz.datex2.test.mockclasses.MockData;
import de.bsvrz.datex2.test.mockclasses.MockDataModel;
import de.bsvrz.datex2.test.mockclasses.MockDavConnection;
import de.bsvrz.datex2.test.mockclasses.MockPids;
import de.bsvrz.datex2.test.mockclasses.MockSystemObject;
import de.bsvrz.dav.daf.main.config.ConfigurationChangeException;
import de.bsvrz.dav.daf.main.config.ObjectSet;


public class CreateMockUmfelddaten extends CreateMockObjects {
  public CreateMockUmfelddaten() {
  }
  
  public CreateMockUmfelddaten(MockDataModel dataModel) {
    super(dataModel);
  }
    
  @Override
  public void create() throws ConfigurationChangeException {
    assert(usedDataModel != null);
    
    MockSystemObject ufdms01 = createUmfelddatenMessstelle("pid.ufdms01", 55432323, 493232454);
    MockSystemObject ufdms02 = createUmfelddatenMessstelle("pid.ufdms02", 55432323, 493232453);
    @SuppressWarnings("unused")
    MockSystemObject ufdms03 = createUmfelddatenMessstelle("pid.ufdms03", 55432323, 493232453);
    
    createUmfelddatensensor(MockPids.TYP_GEFRIERTEMPERATUR, "pid.ufds01.gefriertemperatur", ufdms01);    
    createUmfelddatensensor(MockPids.TYP_GEFRIERTEMPERATUR, "pid.ufds02.gefriertemperatur", ufdms01);    
    createUmfelddatensensor(MockPids.TYP_FAHRBAHNOBERFLAECHENTEMPERATUR, "pid.ufds03.fahrbahnoberfltemperatur", ufdms01);    
    createUmfelddatensensor(MockPids.TYP_FAHRBAHNOBERFLAECHENZUSTAND, "pid.ufds04.fahrbahnoberflzustand", ufdms01);    
    createUmfelddatensensor(MockPids.TYP_LUFTTEMPERATUR, "pid.ufds04.lufttemperatur", ufdms01);    
    createUmfelddatensensor(MockPids.TYP_NIEDERSCHLAGSART, "pid.ufds05.regenart", ufdms01);    
    createUmfelddatensensor(MockPids.TYP_NIEDERSCHLAGSINTENSITAET, "pid.ufds06.regenstark", ufdms01);    
    createUmfelddatensensor(MockPids.TYP_NIEDERSCHLAGSMENGE, "pid.ufds07.regenviel", ufdms01);    
    createUmfelddatensensor(MockPids.TYP_RELATIVELUFTFEUCHTE, "pid.ufds08.dunst", ufdms01);    
    createUmfelddatensensor(MockPids.TYP_RESTSALZ, "pid.ufds09.restsalz", ufdms01);    
    createUmfelddatensensor(MockPids.TYP_SCHNEEHOEHE, "pid.ufds10.skimesser", ufdms01);    
    createUmfelddatensensor(MockPids.TYP_SICHTWEITE, "pid.ufds11.sichtweite", ufdms01);    
    createUmfelddatensensor(MockPids.TYP_TAUPUNKTTEMPERATUR, "pid.ufds12.tautemperatur", ufdms01);    
    createUmfelddatensensor(MockPids.TYP_WASSERFILMDICKE, "pid.ufds13.aquaplan", ufdms01);    
    createUmfelddatensensor(MockPids.TYP_WINDGESCHWINDIGKEITMITTELWERT, "pid.ufds14.avgwind", ufdms01);    
    createUmfelddatensensor(MockPids.TYP_WINDGESCHWINDIGKEITSPITZENWERT, "pid.ufds15.maxwind", ufdms01);    
    createUmfelddatensensor(MockPids.TYP_WINDRICHTUNG, "pid.ufds16.dirwind", ufdms01);    
    createUmfelddatensensor(MockPids.TYP_GEFRIERTEMPERATUR, "pid.ufds03.gefriertemperatur", ufdms02);    
    createUmfelddatensensor(MockPids.TYP_BOSCHUNG, "pid.ufds03.boschung", ufdms02);    
  }
  
  public MockSystemObject createUmfelddatenMessstelle(String pid, int locationX, int locationY) {
    MockSystemObject ufdms01 = createLocatedObject(pid, MockPids.TYP_UMFELDDATENMESSSTELLE, locationX, locationY);

    ufdms01.newObjectSet("UmfeldDatenSensoren");
    
    return ufdms01;
  }
    
  public MockSystemObject createUmfelddatensensor(String typ, String pid, MockSystemObject ufdms) throws ConfigurationChangeException {
    MockSystemObject ufds = new MockSystemObject(pid, typ);
    ufds.setDataModel(usedDataModel);
    usedDataModel.addObject(usedDataModel.getType(typ), ufds);
    verbindeMessstelleUndSensor(ufdms, ufds);
    return ufds;
  }
  
  public void verbindeMessstelleUndSensor(MockSystemObject ufdms, MockSystemObject ufds) throws ConfigurationChangeException {
    if (ufdms != null) {
      ObjectSet objSet = ufdms.getObjectSet("UmfeldDatenSensoren");
      objSet.add(ufds);
    }
  }
  
  public void mockSendDataToAll(MockDavConnection davConnection, Float number, String niederschlagsArt, String zustand) {
    mockSendNumericData(davConnection, MockPids.TYP_FAHRBAHNOBERFLAECHENTEMPERATUR, MockPids.DS_FAHRBAHNOBERFLAECHENTEMPERATUR, number);
    mockSendNumericData(davConnection, MockPids.TYP_GEFRIERTEMPERATUR, MockPids.DS_GEFRIERTEMPERATUR, number);
    mockSendNumericData(davConnection, MockPids.TYP_LUFTTEMPERATUR, MockPids.DS_LUFTTEMPERATUR, number);
    mockSendNumericData(davConnection, MockPids.TYP_NIEDERSCHLAGSINTENSITAET, MockPids.DS_NIEDERSCHLAGSINTENSITAET, number);
    mockSendNumericData(davConnection, MockPids.TYP_NIEDERSCHLAGSMENGE, MockPids.DS_NIEDERSCHLAGSMENGE, number);
    mockSendNumericData(davConnection, MockPids.TYP_RELATIVELUFTFEUCHTE, MockPids.DS_RELATIVELUFTFEUCHTE, number);
    mockSendNumericData(davConnection, MockPids.TYP_RESTSALZ, MockPids.DS_RESTSALZ, number);
    mockSendNumericData(davConnection, MockPids.TYP_SCHNEEHOEHE, MockPids.DS_SCHNEEHOEHE, number);
    mockSendNumericData(davConnection, MockPids.TYP_SICHTWEITE, MockPids.DS_SICHTWEITE, number);
    mockSendNumericData(davConnection, MockPids.TYP_TAUPUNKTTEMPERATUR, MockPids.DS_TAUPUNKTTEMPERATUR, number);
    mockSendNumericData(davConnection, MockPids.TYP_WASSERFILMDICKE, MockPids.DS_WASSERFILMDICKE, number);
    mockSendNumericData(davConnection, MockPids.TYP_WINDGESCHWINDIGKEITMITTELWERT, MockPids.DS_WINDGESCHWINDIGKEITMITTELWERT, number);
    mockSendNumericData(davConnection, MockPids.TYP_WINDGESCHWINDIGKEITSPITZENWERT, MockPids.DS_WINDGESCHWINDIGKEITSPITZENWERT, number);
    mockSendNumericData(davConnection, MockPids.TYP_WINDRICHTUNG, MockPids.DS_WINDRICHTUNG, number);
    mockSendNumericData(davConnection, MockPids.TYP_FAHRBAHNOBERFLAECHENTEMPERATUR, MockPids.DS_FAHRBAHNOBERFLAECHENTEMPERATUR, number);
    mockSendNumericData(davConnection, MockPids.TYP_FAHRBAHNOBERFLAECHENTEMPERATUR, MockPids.DS_FAHRBAHNOBERFLAECHENTEMPERATUR, number);
    mockSendNumericData(davConnection, MockPids.TYP_FAHRBAHNOBERFLAECHENTEMPERATUR, MockPids.DS_FAHRBAHNOBERFLAECHENTEMPERATUR, number);
    mockSendData(davConnection, MockPids.TYP_NIEDERSCHLAGSART, createTextData(MockPids.DS_NIEDERSCHLAGSART, niederschlagsArt));      
    mockSendData(davConnection, MockPids.TYP_FAHRBAHNOBERFLAECHENZUSTAND, createTextData(MockPids.DS_FAHRBAHNOBERFLAECHENZUSTAND, zustand));      
  }

  public void mockSendNumericData(MockDavConnection davConnection, String typ, String item, Number number) {
    mockSendNumericData(davConnection, typ, item, "Wert", number);
  }

  
  public MockData createNumberData(String item, Number value) {
    return createNumberData(item, "Wert", value);
  }

  public MockData createTextData(String item, String value) {
    return createTextData(item, "Wert", value);
  }

  @Override
  public void mockSendValidDataToAll(MockDavConnection davConnection) {
    mockSendDataToAll(davConnection, 0.3f, "Regen", "trocken");
  }

}
