/**
 * SWE KEx-DatexII (de.bsvrz.kex.datex2)
 * 
 * Copyright (C) 2014 TraffGo Road GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * TraffGo Road GmbH
 * Ostwall 11-19
 * 47798 Krefeld
 * Phone: +49-2151-3295058
 * mailto: info@traffgoroad.de
 */

package de.bsvrz.kex.datex2.plugins.definition.impl;
import javax.naming.ConfigurationException;

import de.bsvrz.kex.datex2.allgemein.ObjekteAtgAsp;
import de.bsvrz.kex.datex2.plugins.ausfuehrung.impl.StandardImportPluginAusfuehrung;
import de.bsvrz.kex.datex2.plugins.definition.PluginImport;
import de.bsvrz.kex.datex2.plugins.empfang.impl.MDMEmpfang;
import de.bsvrz.kex.datex2.plugins.konvertierung.impl.ImportVerkehrslage;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Plugin-Definitions-Klasse ("assembly class") fr das Plugin MDM-DaV-001 
 * (proof of concept fr Datenimport/Verkehrslage Messquerschnitte).
 * 
 * @author TraffGo Road GmbH
 * @version $Id$
 */

public class MDM_DaV_001 extends PluginImport {
  /** Typ der zu behandelnden Objekte. */
  private static final String TYP_MESS_QUERSCHNITT = "typ.messQuerschnitt";

  /** Zu behandelnde Attributgruppe. */
  private static final String ATTRIBUT_GRUPPE = "atg.strfallZustand";

  /** Zu behandelnder Aspekt. */
  private String aspekt;
  
  /** Logging. */
  private static final Debug LOGGER = Debug.getLogger();

  /** Standard-Periode des Plugins (in Sekunden). */
  private static final int STANDARD_PERIOD = 60;
  
  /**
   * Konstruktor.
   */
  public MDM_DaV_001() {
    super(STANDARD_PERIOD);
  }


  @Override
  public void initialisiere() throws ConfigurationException {
    LOGGER.finer("Plugin-Klasse " + this.getClass().getName() + " wird initialisiert.");
    aspekt = konfigurationsModul.getString("ImportAspekt");
    objekte.add(new ObjekteAtgAsp(davConnection, TYP_MESS_QUERSCHNITT, ATTRIBUT_GRUPPE, aspekt));
    initialisiereAbhaengigkeiten();
  }
  
  @Override
  public void instantiiereEmpfangsMethode() {
    empfangsMethode = new MDMEmpfang(this);
  }

  @Override
  public void instantiiereDavKonvertierungDynamisch() {
    this.davKonvertierungDynamisch = new ImportVerkehrslage(this);    
  }

  @Override
  public void instantiierePluginAusfuehrungsAlgorithmus() {
    this.pluginAusfuehrungsAlgorithmus = new StandardImportPluginAusfuehrung(this);    
  }

}
